@@ -99,14 +99,14 @@ class StreamFilterChild final : public PStreamFilterChild,
   IPCResult RecvResumed();
   IPCResult RecvFlushData();
 
-  virtual void DeallocPStreamFilterChild() override;
+  virtual void ActorDealloc() override;
 
   void SetStreamFilter(StreamFilter* aStreamFilter) {
     mStreamFilter = aStreamFilter;
   }
 
  private:
-  ~StreamFilterChild() {}
+  ~StreamFilterChild() = default;
 
   void SetNextState();
 
