@@ -5,50 +5,23 @@ ChromeUtils.import("resource://gre/modules/TelemetryEnvironment.jsm", this);
 ChromeUtils.import("resource://normandy/lib/PreferenceExperiments.jsm", this);
 ChromeUtils.import("resource://normandy/lib/CleanupManager.jsm", this);
 ChromeUtils.import("resource://normandy/lib/TelemetryEvents.jsm", this);
+ChromeUtils.import("resource://normandy/lib/NormandyUtils.jsm", this);
+ChromeUtils.import("resource://testing-common/NormandyTestUtils.jsm", this);
 
 // Save ourselves some typing
 const { withMockExperiments } = PreferenceExperiments;
 const DefaultPreferences = new Preferences({ defaultBranch: true });
 const startupPrefs = "app.normandy.startupExperimentPrefs";
+const { preferenceStudyFactory } = NormandyTestUtils.factories;
 
-function experimentFactory(attrs) {
-  const defaultPref = {
-    "fake.preference": {},
-  };
-  const defaultPrefInfo = {
-    preferenceValue: "fakevalue",
-    preferenceType: "string",
-    previousPreferenceValue: "oldfakevalue",
-    preferenceBranchType: "default",
-  };
-  const preferences = {};
-  for (const [prefName, prefInfo] of Object.entries(
-    attrs.preferences || defaultPref
-  )) {
-    preferences[prefName] = { ...defaultPrefInfo, ...prefInfo };
-  }
-
-  return Object.assign(
-    {
-      name: "fakename",
-      branch: "fakebranch",
-      expired: false,
-      lastSeen: new Date().toJSON(),
-      experimentType: "exp",
-    },
-    attrs,
-    {
-      preferences,
-    }
-  );
-}
+const NOW = new Date();
 
 const mockV1Data = {
   hypothetical_experiment: {
     name: "hypothetical_experiment",
     branch: "hypo_1",
     expired: false,
-    lastSeen: new Date().toJSON(),
+    lastSeen: NOW.toJSON(),
     preferenceName: "some.pref",
     preferenceValue: 2,
     preferenceType: "integer",
@@ -60,7 +33,7 @@ const mockV1Data = {
     name: "another_experiment",
     branch: "another_4",
     expired: true,
-    lastSeen: new Date().toJSON(),
+    lastSeen: NOW.toJSON(),
     preferenceName: "another.pref",
     preferenceValue: true,
     preferenceType: "boolean",
@@ -71,13 +44,41 @@ const mockV1Data = {
 };
 
 const mockV2Data = {
-  __version: 2,
   experiments: {
     hypothetical_experiment: {
       name: "hypothetical_experiment",
       branch: "hypo_1",
       expired: false,
-      lastSeen: mockV1Data.hypothetical_experiment.lastSeen,
+      lastSeen: NOW.toJSON(),
+      preferenceName: "some.pref",
+      preferenceValue: 2,
+      preferenceType: "integer",
+      previousPreferenceValue: 1,
+      preferenceBranchType: "user",
+      experimentType: "exp",
+    },
+    another_experiment: {
+      name: "another_experiment",
+      branch: "another_4",
+      expired: true,
+      lastSeen: NOW.toJSON(),
+      preferenceName: "another.pref",
+      preferenceValue: true,
+      preferenceType: "boolean",
+      previousPreferenceValue: false,
+      preferenceBranchType: "default",
+      experimentType: "exp",
+    },
+  },
+};
+
+const mockV3Data = {
+  experiments: {
+    hypothetical_experiment: {
+      name: "hypothetical_experiment",
+      branch: "hypo_1",
+      expired: false,
+      lastSeen: NOW.toJSON(),
       preferences: {
         "some.pref": {
           preferenceValue: 2,
@@ -92,7 +93,7 @@ const mockV2Data = {
       name: "another_experiment",
       branch: "another_4",
       expired: true,
-      lastSeen: mockV1Data.another_experiment.lastSeen,
+      lastSeen: NOW.toJSON(),
       preferences: {
         "another.pref": {
           preferenceValue: true,
@@ -106,15 +107,14 @@ const mockV2Data = {
   },
 };
 
-const MIGRATED_DATA = {
-  __version: 3,
+const mockV4Data = {
   experiments: {
     hypothetical_experiment: {
       name: "hypothetical_experiment",
       branch: "hypo_1",
       actionName: "SinglePreferenceExperimentAction",
       expired: false,
-      lastSeen: mockV1Data.hypothetical_experiment.lastSeen,
+      lastSeen: NOW.toJSON(),
       preferences: {
         "some.pref": {
           preferenceValue: 2,
@@ -130,7 +130,7 @@ const MIGRATED_DATA = {
       branch: "another_4",
       actionName: "SinglePreferenceExperimentAction",
       expired: true,
-      lastSeen: mockV1Data.another_experiment.lastSeen,
+      lastSeen: NOW.toJSON(),
       preferences: {
         "another.pref": {
           preferenceValue: true,
@@ -144,57 +144,169 @@ const MIGRATED_DATA = {
   },
 };
 
-add_task(function migrateStorage_migrates_to_new_format() {
-  let mockJsonFile = {
-    // Deep clone the data in case migrateStorage mutates it.
-    data: JSON.parse(JSON.stringify(mockV1Data)),
+const mockV5Data = {
+  experiments: {
+    hypothetical_experiment: {
+      slug: "hypothetical_experiment",
+      branch: "hypo_1",
+      actionName: "SinglePreferenceExperimentAction",
+      expired: false,
+      lastSeen: NOW.toJSON(),
+      preferences: {
+        "some.pref": {
+          preferenceValue: 2,
+          preferenceType: "integer",
+          previousPreferenceValue: 1,
+          preferenceBranchType: "user",
+        },
+      },
+      experimentType: "exp",
+    },
+    another_experiment: {
+      slug: "another_experiment",
+      branch: "another_4",
+      actionName: "SinglePreferenceExperimentAction",
+      expired: true,
+      lastSeen: NOW.toJSON(),
+      preferences: {
+        "another.pref": {
+          preferenceValue: true,
+          preferenceType: "boolean",
+          previousPreferenceValue: false,
+          preferenceBranchType: "default",
+        },
+      },
+      experimentType: "exp",
+    },
+  },
+};
+
+const migrationsInfo = [
+  {
+    migration: PreferenceExperiments.migrations.migration01MoveExperiments,
+    dataBefore: mockV1Data,
+    dataAfter: mockV2Data,
+  },
+  {
+    migration: PreferenceExperiments.migrations.migration02MultiPreference,
+    dataBefore: mockV2Data,
+    dataAfter: mockV3Data,
+  },
+  {
+    migration: PreferenceExperiments.migrations.migration03AddActionName,
+    dataBefore: mockV3Data,
+    dataAfter: mockV4Data,
+  },
+  {
+    migration: PreferenceExperiments.migrations.migration04RenameNameToSlug,
+    dataBefore: mockV4Data,
+    dataAfter: mockV5Data,
+  },
+  // Migration 5 is not a simple data migration. This style of tests does not apply to it.
+];
+
+/**
+ * Make a mock `JsonFile` object with a no-op `saveSoon` method and a deep copy
+ * of the data passed.
+ * @param {Object} data the data in the store
+ */
+function makeMockJsonFile(data = {}) {
+  return {
+    // Deep clone the data in case migrations mutate it.
+    data: JSON.parse(JSON.stringify(data)),
+    saveSoon: () => {},
   };
-  migrateStorage(mockJsonFile);
-  Assert.deepEqual(mockJsonFile.data, MIGRATED_DATA);
+}
 
-  mockJsonFile = {
-    data: JSON.parse(JSON.stringify(mockV2Data)),
-  };
-  migrateStorage(mockJsonFile);
-  Assert.deepEqual(mockJsonFile.data, MIGRATED_DATA);
-});
-
-add_task(function migrateStorage_keeps_actionNames() {
-  let mockData = JSON.parse(JSON.stringify(mockV2Data));
-  mockData.experiments.another_experiment.actionName = "SomeOldAction";
-  const mockJsonFile = {
-    data: mockData,
-  };
-  // Output should be the same as MIGRATED_DATA, but preserving the action.
-  const migratedData = JSON.parse(JSON.stringify(MIGRATED_DATA));
-  migratedData.experiments.another_experiment.actionName = "SomeOldAction";
-
-  migrateStorage(mockJsonFile);
-  Assert.deepEqual(mockJsonFile.data, migratedData);
-});
-
-add_task(function migrateStorage_is_idempotent() {
-  for (const [name, mockOldData] of [["v1", mockV1Data], ["v2", mockV2Data]]) {
-    const mockJsonFileOnce = {
-      data: JSON.parse(JSON.stringify(mockOldData)),
-    };
-    const mockJsonFileTwice = {
-      data: JSON.parse(JSON.stringify(mockOldData)),
-    };
-    migrateStorage(mockJsonFileOnce);
-    migrateStorage(mockJsonFileTwice);
-    migrateStorage(mockJsonFileTwice);
+/** Test that each migration results in the expected data */
+add_task(async function test_migrations() {
+  for (const { migration, dataAfter, dataBefore } of migrationsInfo) {
+    let mockJsonFile = makeMockJsonFile(dataBefore);
+    await migration(mockJsonFile);
     Assert.deepEqual(
-      mockJsonFileOnce,
-      mockJsonFileTwice,
-      "migrating data twice should be idempotent for " + name
+      mockJsonFile.data,
+      dataAfter,
+      `Migration ${migration.name} should result in the expected data`
     );
   }
 });
 
+add_task(async function migrations_are_idempotent() {
+  for (const { migration, dataBefore } of migrationsInfo) {
+    const mockJsonFileOnce = makeMockJsonFile(dataBefore);
+    const mockJsonFileTwice = makeMockJsonFile(dataBefore);
+    await migration(mockJsonFileOnce);
+    await migration(mockJsonFileTwice);
+    await migration(mockJsonFileTwice);
+    Assert.deepEqual(
+      mockJsonFileOnce.data,
+      mockJsonFileTwice.data,
+      "migrating data twice should be idempotent for " + migration.name
+    );
+  }
+});
+
+add_task(async function migration03KeepsActionName() {
+  let mockData = JSON.parse(JSON.stringify(mockV3Data));
+  mockData.experiments.another_experiment.actionName = "SomeOldAction";
+  const mockJsonFile = makeMockJsonFile(mockData);
+  // Output should be the same as mockV4Data, but preserving the action.
+  const migratedData = JSON.parse(JSON.stringify(mockV4Data));
+  migratedData.experiments.another_experiment.actionName = "SomeOldAction";
+
+  await PreferenceExperiments.migrations.migration03AddActionName(mockJsonFile);
+  Assert.deepEqual(mockJsonFile.data, migratedData);
+});
+
+// Test that migration 5 works as expected
+decorate_task(
+  PreferenceExperiments.withMockExperiments([
+    NormandyTestUtils.factories.preferenceStudyFactory({
+      actionName: "PreferenceExperimentAction",
+      expired: false,
+    }),
+    NormandyTestUtils.factories.preferenceStudyFactory({
+      actionName: "SinglePreferenceExperimentAction",
+      expired: false,
+    }),
+  ]),
+  async function migration05Works([expKeep, expExpire]) {
+    // pre check
+    const activeSlugsBefore = (await PreferenceExperiments.getAllActive()).map(
+      e => e.slug
+    );
+    Assert.deepEqual(
+      activeSlugsBefore,
+      [expKeep.slug, expExpire.slug],
+      "Both experiments should be present and active before the migration"
+    );
+
+    // run the migration
+    await PreferenceExperiments.migrations.migration05RemoveOldAction();
+
+    // verify behavior
+    const activeSlugsAfter = (await PreferenceExperiments.getAllActive()).map(
+      e => e.slug
+    );
+    Assert.deepEqual(
+      activeSlugsAfter,
+      [expKeep.slug],
+      "The single pref experiment should be ended by the migration"
+    );
+    const allSlugsAfter = (await PreferenceExperiments.getAll()).map(
+      e => e.slug
+    );
+    Assert.deepEqual(
+      allSlugsAfter,
+      [expKeep.slug, expExpire.slug],
+      "Both experiments should still exist after the migration"
+    );
+  }
+);
+
 // clearAllExperimentStorage
 decorate_task(
-  withMockExperiments([experimentFactory({ name: "test" })]),
+  withMockExperiments([preferenceStudyFactory({ slug: "test" })]),
   async function(experiments) {
     ok(await PreferenceExperiments.has("test"), "Mock experiment is detected.");
     await PreferenceExperiments.clearAllExperimentStorage();
@@ -207,12 +319,12 @@ decorate_task(
 
 // start should throw if an experiment with the given name already exists
 decorate_task(
-  withMockExperiments([experimentFactory({ name: "test" })]),
+  withMockExperiments([preferenceStudyFactory({ slug: "test" })]),
   withSendEventStub,
   async function(experiments, sendEventStub) {
     await Assert.rejects(
       PreferenceExperiments.start({
-        name: "test",
+        slug: "test",
         actionName: "SomeAction",
         branch: "branch",
         preferences: {
@@ -237,8 +349,8 @@ decorate_task(
 // preferences are active
 decorate_task(
   withMockExperiments([
-    experimentFactory({
-      name: "test",
+    preferenceStudyFactory({
+      slug: "test",
       preferences: { "fake.preferenceinteger": {} },
     }),
   ]),
@@ -246,7 +358,7 @@ decorate_task(
   async function(experiments, sendEventStub) {
     await Assert.rejects(
       PreferenceExperiments.start({
-        name: "different",
+        slug: "different",
         actionName: "SomeAction",
         branch: "branch",
         preferences: {
@@ -284,7 +396,7 @@ decorate_task(withMockExperiments(), withSendEventStub, async function(
 ) {
   await Assert.rejects(
     PreferenceExperiments.start({
-      name: "test",
+      slug: "test",
       actionName: "SomeAction",
       branch: "branch",
       preferences: {
@@ -323,7 +435,7 @@ decorate_task(
     mockPreferences.set("fake.preferenceinteger", 101, "user");
 
     const experiment = {
-      name: "test",
+      slug: "test",
       actionName: "SomeAction",
       branch: "branch",
       preferences: {
@@ -347,7 +459,7 @@ decorate_task(
     );
 
     const expectedExperiment = {
-      name: "test",
+      slug: "test",
       branch: "branch",
       expired: false,
       preferences: {
@@ -419,7 +531,7 @@ decorate_task(
     mockPreferences.set("fake.preference", "oldvalue", "user");
 
     const experiment = {
-      name: "test",
+      slug: "test",
       actionName: "SomeAction",
       branch: "branch",
       preferences: {
@@ -437,7 +549,7 @@ decorate_task(
     );
 
     const expectedExperiment = {
-      name: "test",
+      slug: "test",
       branch: "branch",
       expired: false,
       preferences: {
@@ -483,7 +595,7 @@ decorate_task(withMockPreferences, withSendEventStub, async function(
 
   await Assert.rejects(
     PreferenceExperiments.start({
-      name: "test",
+      slug: "test",
       actionName: "SomeAction",
       branch: "branch",
       preferences: {
@@ -730,7 +842,9 @@ decorate_task(withMockExperiments(), async function() {
 // markLastSeen should update the lastSeen date
 const oldDate = new Date(1988, 10, 1).toJSON();
 decorate_task(
-  withMockExperiments([experimentFactory({ name: "test", lastSeen: oldDate })]),
+  withMockExperiments([
+    preferenceStudyFactory({ slug: "test", lastSeen: oldDate }),
+  ]),
   async function([experiment]) {
     await PreferenceExperiments.markLastSeen("test");
     Assert.notEqual(
@@ -764,7 +878,9 @@ decorate_task(withMockExperiments(), withSendEventStub, async function(
 
 // stop should throw if the experiment is already expired
 decorate_task(
-  withMockExperiments([experimentFactory({ name: "test", expired: true })]),
+  withMockExperiments([
+    preferenceStudyFactory({ slug: "test", expired: true }),
+  ]),
   withSendEventStub,
   async function(experiments, sendEventStub) {
     await Assert.rejects(
@@ -788,8 +904,8 @@ decorate_task(
 // preference value.
 decorate_task(
   withMockExperiments([
-    experimentFactory({
-      name: "test",
+    preferenceStudyFactory({
+      slug: "test",
       expired: false,
       branch: "fakebranch",
       preferences: {
@@ -866,8 +982,8 @@ decorate_task(
 // stop should also support user pref experiments
 decorate_task(
   withMockExperiments([
-    experimentFactory({
-      name: "test",
+    preferenceStudyFactory({
+      slug: "test",
       expired: false,
       preferences: {
         "fake.preference": {
@@ -915,8 +1031,8 @@ decorate_task(
 // stop should remove a preference that had no value prior to an experiment for user prefs
 decorate_task(
   withMockExperiments([
-    experimentFactory({
-      name: "test",
+    preferenceStudyFactory({
+      slug: "test",
       expired: false,
       preferences: {
         "fake.preference": {
@@ -946,8 +1062,8 @@ decorate_task(
 // stop should not modify a preference if resetValue is false
 decorate_task(
   withMockExperiments([
-    experimentFactory({
-      name: "test",
+    preferenceStudyFactory({
+      slug: "test",
       expired: false,
       branch: "fakebranch",
       preferences: {
@@ -1006,23 +1122,23 @@ decorate_task(withMockExperiments(), async function() {
 
 // get
 decorate_task(
-  withMockExperiments([experimentFactory({ name: "test" })]),
+  withMockExperiments([preferenceStudyFactory({ slug: "test" })]),
   async function(experiments) {
     const experiment = await PreferenceExperiments.get("test");
-    is(experiment.name, "test", "get fetches the correct experiment");
+    is(experiment.slug, "test", "get fetches the correct experiment");
 
     // Modifying the fetched experiment must not edit the data source.
-    experiment.name = "othername";
+    experiment.slug = "othername";
     const refetched = await PreferenceExperiments.get("test");
-    is(refetched.name, "test", "get returns a copy of the experiment");
+    is(refetched.slug, "test", "get returns a copy of the experiment");
   }
 );
 
 // get all
 decorate_task(
   withMockExperiments([
-    experimentFactory({ name: "experiment1", disabled: false }),
-    experimentFactory({ name: "experiment2", disabled: true }),
+    preferenceStudyFactory({ slug: "experiment1", disabled: false }),
+    preferenceStudyFactory({ slug: "experiment2", disabled: true }),
   ]),
   async function testGetAll([experiment1, experiment2]) {
     const fetchedExperiments = await PreferenceExperiments.getAll();
@@ -1032,12 +1148,12 @@ decorate_task(
       "getAll returns a list of all stored experiments"
     );
     Assert.deepEqual(
-      fetchedExperiments.find(e => e.name === "experiment1"),
+      fetchedExperiments.find(e => e.slug === "experiment1"),
       experiment1,
       "getAll returns a list with the correct experiments"
     );
     const fetchedExperiment2 = fetchedExperiments.find(
-      e => e.name === "experiment2"
+      e => e.slug === "experiment2"
     );
     Assert.deepEqual(
       fetchedExperiment2,
@@ -1045,9 +1161,9 @@ decorate_task(
       "getAll returns a list with the correct experiments, including disabled ones"
     );
 
-    fetchedExperiment2.name = "othername";
+    fetchedExperiment2.slug = "otherslug";
     is(
-      experiment2.name,
+      experiment2.slug,
       "experiment2",
       "getAll returns copies of the experiments"
     );
@@ -1057,12 +1173,12 @@ decorate_task(
 // get all active
 decorate_task(
   withMockExperiments([
-    experimentFactory({
-      name: "active",
+    preferenceStudyFactory({
+      slug: "active",
       expired: false,
     }),
-    experimentFactory({
-      name: "inactive",
+    preferenceStudyFactory({
+      slug: "inactive",
       expired: true,
     }),
   ]),
@@ -1075,7 +1191,7 @@ decorate_task(
       "getAllActive only returns active experiments"
     );
 
-    allActiveExperiments[0].name = "newfakename";
+    allActiveExperiments[0].slug = "newfakename";
     allActiveExperiments = await PreferenceExperiments.getAllActive();
     Assert.notEqual(
       allActiveExperiments,
@@ -1087,7 +1203,7 @@ decorate_task(
 
 // has
 decorate_task(
-  withMockExperiments([experimentFactory({ name: "test" })]),
+  withMockExperiments([preferenceStudyFactory({ slug: "test" })]),
   async function(experiments) {
     ok(
       await PreferenceExperiments.has("test"),
@@ -1103,14 +1219,14 @@ decorate_task(
 // init should register telemetry experiments
 decorate_task(
   withMockExperiments([
-    experimentFactory({
-      name: "test",
+    preferenceStudyFactory({
+      slug: "test",
       branch: "branch",
       preferences: {
         "fake.pref": {
           preferenceValue: "experiment value",
-          expired: false,
           preferenceBranchType: "default",
+          preferenceType: "string",
         },
       },
     }),
@@ -1127,7 +1243,10 @@ decorate_task(
     mockPreferences.set("fake.pref", "experiment value");
     await PreferenceExperiments.init();
     ok(
-      setActiveStub.calledWith("test", "branch", { type: "normandy-exp" }),
+      setActiveStub.calledWith("test", "branch", {
+        type: "normandy-exp",
+        enrollmentId: experiments[0].enrollmentId,
+      }),
       "Experiment is registered by init"
     );
   }
@@ -1136,12 +1255,13 @@ decorate_task(
 // init should use the provided experiment type
 decorate_task(
   withMockExperiments([
-    experimentFactory({
-      name: "test",
+    preferenceStudyFactory({
+      slug: "test",
       branch: "branch",
       preferences: {
         "fake.pref": {
           preferenceValue: "experiment value",
+          preferenceType: "string",
         },
       },
       experimentType: "pref-test",
@@ -1161,6 +1281,7 @@ decorate_task(
     ok(
       setActiveStub.calledWith("test", "branch", {
         type: "normandy-pref-test",
+        enrollmentId: sinon.match(NormandyTestUtils.isUuid),
       }),
       "init should use the provided experiment type"
     );
@@ -1179,8 +1300,8 @@ decorate_task(
     setInactiveStub,
     sendEventStub
   ) {
-    await PreferenceExperiments.start({
-      name: "test",
+    let { enrollmentId } = await PreferenceExperiments.start({
+      slug: "test",
       actionName: "SomeAction",
       branch: "branch",
       preferences: {
@@ -1192,9 +1313,14 @@ decorate_task(
       },
     });
 
+    ok(
+      NormandyTestUtils.isUuid(enrollmentId),
+      "Experiment should have a UUID enrollmentId"
+    );
+
     Assert.deepEqual(
       setActiveStub.getCall(0).args,
-      ["test", "branch", { type: "normandy-exp" }],
+      ["test", "branch", { type: "normandy-exp", enrollmentId }],
       "Experiment is registered by start()"
     );
     await PreferenceExperiments.stop("test", { reason: "test-reason" });
@@ -1212,6 +1338,7 @@ decorate_task(
         {
           experimentType: "exp",
           branch: "branch",
+          enrollmentId,
         },
       ],
       [
@@ -1222,6 +1349,7 @@ decorate_task(
           reason: "test-reason",
           didResetValue: "true",
           branch: "branch",
+          enrollmentId,
         },
       ],
     ]);
@@ -1240,8 +1368,8 @@ decorate_task(
     setInactiveStub,
     sendEventStub
   ) {
-    await PreferenceExperiments.start({
-      name: "test",
+    const { enrollmentId } = await PreferenceExperiments.start({
+      slug: "test",
       actionName: "SomeAction",
       branch: "branch",
       preferences: {
@@ -1256,7 +1384,7 @@ decorate_task(
 
     Assert.deepEqual(
       setActiveStub.getCall(0).args,
-      ["test", "branch", { type: "normandy-pref-test" }],
+      ["test", "branch", { type: "normandy-pref-test", enrollmentId }],
       "start() should register the experiment with the provided type"
     );
 
@@ -1268,6 +1396,7 @@ decorate_task(
         {
           experimentType: "pref-test",
           branch: "branch",
+          enrollmentId,
         },
       ],
     ]);
@@ -1281,7 +1410,11 @@ decorate_task(
 // Experiments shouldn't be recorded by init() in telemetry if they are expired
 decorate_task(
   withMockExperiments([
-    experimentFactory({ name: "expired", branch: "branch", expired: true }),
+    preferenceStudyFactory({
+      slug: "expired",
+      branch: "branch",
+      expired: true,
+    }),
   ]),
   withStub(TelemetryEnvironment, "setExperimentActive"),
   async function testInitTelemetryExpired(experiments, setActiveStub) {
@@ -1293,11 +1426,12 @@ decorate_task(
 // Experiments should end if the preference has been changed when init() is called
 decorate_task(
   withMockExperiments([
-    experimentFactory({
-      name: "test",
+    preferenceStudyFactory({
+      slug: "test",
       preferences: {
         "fake.preference": {
           preferenceValue: "experiment value",
+          preferenceType: "string",
         },
       },
     }),
@@ -1332,11 +1466,13 @@ decorate_task(
 // init should register an observer for experiments
 decorate_task(
   withMockExperiments([
-    experimentFactory({
-      name: "test",
+    preferenceStudyFactory({
+      slug: "test",
       preferences: {
         "fake.preference": {
           preferenceValue: "experiment value",
+          preferenceType: "string",
+          previousPreferenceValue: "oldfakevalue",
         },
       },
     }),
@@ -1382,27 +1518,30 @@ decorate_task(
 // saveStartupPrefs
 decorate_task(
   withMockExperiments([
-    experimentFactory({
-      name: "char",
+    preferenceStudyFactory({
+      slug: "char",
       preferences: {
         "fake.char": {
           preferenceValue: "string",
+          preferenceType: "string",
         },
       },
     }),
-    experimentFactory({
-      name: "int",
+    preferenceStudyFactory({
+      slug: "int",
       preferences: {
         "fake.int": {
           preferenceValue: 2,
+          preferenceType: "int",
         },
       },
     }),
-    experimentFactory({
-      name: "bool",
+    preferenceStudyFactory({
+      slug: "bool",
       preferences: {
         "fake.bool": {
           preferenceValue: true,
+          preferenceType: "boolean",
         },
       },
     }),
@@ -1436,8 +1575,8 @@ decorate_task(
 // saveStartupPrefs errors for invalid pref type
 decorate_task(
   withMockExperiments([
-    experimentFactory({
-      name: "test",
+    preferenceStudyFactory({
+      slug: "test",
       preferences: {
         "fake.invalidValue": {
           preferenceValue: new Date(),
@@ -1457,20 +1596,22 @@ decorate_task(
 // saveStartupPrefs should not store values for user-branch recipes
 decorate_task(
   withMockExperiments([
-    experimentFactory({
-      name: "defaultBranchRecipe",
+    preferenceStudyFactory({
+      slug: "defaultBranchRecipe",
       preferences: {
         "fake.default": {
           preferenceValue: "experiment value",
+          preferenceType: "string",
           preferenceBranchType: "default",
         },
       },
     }),
-    experimentFactory({
-      name: "userBranchRecipe",
+    preferenceStudyFactory({
+      slug: "userBranchRecipe",
       preferences: {
         "fake.user": {
           preferenceValue: "experiment value",
+          preferenceType: "string",
           preferenceBranchType: "user",
         },
       },
@@ -1520,7 +1661,7 @@ decorate_task(
 
     // start an experiment
     await PreferenceExperiments.start({
-      name: "test",
+      slug: "test",
       actionName: "SomeAction",
       branch: "branch",
       preferences: {
@@ -1574,7 +1715,7 @@ decorate_task(
 
     // start an experiment
     await PreferenceExperiments.start({
-      name: "test",
+      slug: "test",
       actionName: "SomeAction",
       branch: "branch",
       preferences: {
@@ -1616,7 +1757,15 @@ decorate_task(
 // stop should pass "unknown" to telemetry event for `reason` if none is specified
 decorate_task(
   withMockExperiments([
-    experimentFactory({ name: "test", preferences: { "fake.preference": {} } }),
+    preferenceStudyFactory({
+      slug: "test",
+      preferences: {
+        "fake.preference": {
+          preferenceValue: "experiment value",
+          preferenceType: "string",
+        },
+      },
+    }),
   ]),
   withMockPreferences,
   withStub(PreferenceExperiments, "stopObserver"),
@@ -1640,13 +1789,23 @@ decorate_task(
 // stop should pass along the value for resetValue to Telemetry Events as didResetValue
 decorate_task(
   withMockExperiments([
-    experimentFactory({
-      name: "test1",
-      preferences: { "fake.preference1": {} },
+    preferenceStudyFactory({
+      slug: "test1",
+      preferences: {
+        "fake.preference1": {
+          preferenceValue: "experiment value",
+          preferenceType: "string",
+        },
+      },
     }),
-    experimentFactory({
-      name: "test2",
-      preferences: { "fake.preference2": {} },
+    preferenceStudyFactory({
+      slug: "test2",
+      preferences: {
+        "fake.preference2": {
+          preferenceValue: "experiment value",
+          preferenceType: "string",
+        },
+      },
     }),
   ]),
   withMockPreferences,
@@ -1684,8 +1843,8 @@ decorate_task(
   withMockPreferences,
   withSendEventStub,
   withMockExperiments([
-    experimentFactory({
-      name: "test",
+    preferenceStudyFactory({
+      slug: "test",
       expired: false,
       branch: "fakebranch",
       preferences: {
@@ -1719,8 +1878,13 @@ decorate_task(
     // setting the preference on the user branch should trigger the observer to stop the experiment
     mockPreferences.set("fake.preference", "uservalue", "user");
 
-    // let the event loop tick to run the observer
-    await Promise.resolve();
+    // Wait until the change is processed
+    await TestUtils.topicObserved(
+      "normandy:preference-experiment:stopped",
+      (subject, message) => {
+        return message == "test";
+      }
+    );
 
     sendEventStub.assertEvents([
       [
@@ -1731,6 +1895,7 @@ decorate_task(
           didResetValue: "false",
           reason: "user-preference-changed",
           branch: "fakebranch",
+          enrollmentId: mockExperiments[0].enrollmentId,
         },
       ],
     ]);
