@@ -3,12 +3,14 @@
 ChromeUtils.import("resource://gre/modules/Services.jsm", this);
 ChromeUtils.import("resource://gre/modules/Preferences.jsm", this);
 ChromeUtils.import("resource://gre/modules/TelemetryEnvironment.jsm", this);
+ChromeUtils.import("resource://normandy/actions/BaseAction.jsm", this);
 ChromeUtils.import(
   "resource://normandy/actions/PreferenceRolloutAction.jsm",
   this
 );
 ChromeUtils.import("resource://normandy/lib/PreferenceRollouts.jsm", this);
 ChromeUtils.import("resource://normandy/lib/TelemetryEvents.jsm", this);
+ChromeUtils.import("resource://testing-common/NormandyTestUtils.jsm", this);
 
 // Test that a simple recipe enrolls as expected
 decorate_task(
@@ -32,7 +34,7 @@ decorate_task(
     };
 
     const action = new PreferenceRolloutAction();
-    await action.runRecipe(recipe);
+    await action.processRecipe(recipe, BaseAction.suitability.FILTER_MATCH);
     await action.finalize();
     is(action.lastError, null, "lastError should be null");
 
@@ -71,8 +73,9 @@ decorate_task(
     );
 
     // rollout was stored
+    let rollouts = await PreferenceRollouts.getAll();
     Assert.deepEqual(
-      await PreferenceRollouts.getAll(),
+      rollouts,
       [
         {
           slug: "test-rollout",
@@ -86,17 +89,29 @@ decorate_task(
               previousValue: null,
             },
           ],
+          enrollmentId: rollouts[0].enrollmentId,
         },
       ],
       "Rollout should be stored in db"
     );
+    ok(
+      NormandyTestUtils.isUuid(rollouts[0].enrollmentId),
+      "Rollout should have a UUID enrollmentId"
+    );
 
     sendEventStub.assertEvents([
-      ["enroll", "preference_rollout", recipe.arguments.slug],
+      [
+        "enroll",
+        "preference_rollout",
+        recipe.arguments.slug,
+        { enrollmentId: rollouts[0].enrollmentId },
+      ],
     ]);
-    Assert.deepEqual(
-      setExperimentActiveStub.args,
-      [["test-rollout", "active", { type: "normandy-prefrollout" }]],
+    ok(
+      setExperimentActiveStub.calledWithExactly("test-rollout", "active", {
+        type: "normandy-prefrollout",
+        enrollmentId: rollouts[0].enrollmentId,
+      }),
       "a telemetry experiment should be activated"
     );
 
@@ -125,7 +140,7 @@ decorate_task(
     };
 
     let action = new PreferenceRolloutAction();
-    await action.runRecipe(recipe);
+    await action.processRecipe(recipe, BaseAction.suitability.FILTER_MATCH);
     await action.finalize();
     is(action.lastError, null, "lastError should be null");
 
@@ -152,7 +167,7 @@ decorate_task(
       { preferenceName: "test.pref3", value: 2 },
     ];
     action = new PreferenceRolloutAction();
-    await action.runRecipe(recipe);
+    await action.processRecipe(recipe, BaseAction.suitability.FILTER_MATCH);
     await action.finalize();
     is(action.lastError, null, "lastError should be null");
 
@@ -182,8 +197,9 @@ decorate_task(
     );
 
     // rollout in the DB has been updated
+    const rollouts = await PreferenceRollouts.getAll();
     Assert.deepEqual(
-      await PreferenceRollouts.getAll(),
+      rollouts,
       [
         {
           slug: "test-rollout",
@@ -198,12 +214,17 @@ decorate_task(
     );
 
     sendEventStub.assertEvents([
-      ["enroll", "preference_rollout", "test-rollout"],
+      [
+        "enroll",
+        "preference_rollout",
+        "test-rollout",
+        { enrollmentId: rollouts[0].enrollmentId },
+      ],
       [
         "update",
         "preference_rollout",
         "test-rollout",
-        { previousState: "active" },
+        { previousState: "active", enrollmentId: rollouts[0].enrollmentId },
       ],
     ]);
 
@@ -226,6 +247,7 @@ decorate_task(
       preferences: [
         { preferenceName: "test.pref", value: 1, previousValue: 1 },
       ],
+      enrollmentId: "test-enrollment-id",
     });
 
     let recipe = {
@@ -237,7 +259,7 @@ decorate_task(
     };
 
     const action = new PreferenceRolloutAction();
-    await action.runRecipe(recipe);
+    await action.processRecipe(recipe, BaseAction.suitability.FILTER_MATCH);
     await action.finalize();
     is(action.lastError, null, "lastError should be null");
 
@@ -249,8 +271,9 @@ decorate_task(
     );
 
     // rollout in the DB has been ungraduated
+    const rollouts = await PreferenceRollouts.getAll();
     Assert.deepEqual(
-      await PreferenceRollouts.getAll(),
+      rollouts,
       [
         {
           slug: "test-rollout",
@@ -268,7 +291,7 @@ decorate_task(
         "update",
         "preference_rollout",
         "test-rollout",
-        { previousState: "graduated" },
+        { previousState: "graduated", enrollmentId: "test-enrollment-id" },
       ],
     ]);
 
@@ -306,14 +329,14 @@ decorate_task(
 
     // running both in the same session
     let action = new PreferenceRolloutAction();
-    await action.runRecipe(recipe1);
-    await action.runRecipe(recipe2);
+    await action.processRecipe(recipe1, BaseAction.suitability.FILTER_MATCH);
+    await action.processRecipe(recipe2, BaseAction.suitability.FILTER_MATCH);
     await action.finalize();
     is(action.lastError, null, "lastError should be null");
 
     // running recipe2 in a separate session shouldn't change things
     action = new PreferenceRolloutAction();
-    await action.runRecipe(recipe2);
+    await action.processRecipe(recipe2, BaseAction.suitability.FILTER_MATCH);
     await action.finalize();
     is(action.lastError, null, "lastError should be null");
 
@@ -350,8 +373,9 @@ decorate_task(
     );
 
     // only successful rollout was stored
+    const rollouts = await PreferenceRollouts.getAll();
     Assert.deepEqual(
-      await PreferenceRollouts.getAll(),
+      rollouts,
       [
         {
           slug: "test-rollout-1",
@@ -360,6 +384,7 @@ decorate_task(
             { preferenceName: "test.pref1", value: 1, previousValue: null },
             { preferenceName: "test.pref2", value: 1, previousValue: null },
           ],
+          enrollmentId: rollouts[0].enrollmentId,
         },
       ],
       "Only recipe1's rollout should be stored in db"
@@ -403,7 +428,7 @@ decorate_task(
     };
 
     const action = new PreferenceRolloutAction();
-    await action.runRecipe(recipe);
+    await action.processRecipe(recipe, BaseAction.suitability.FILTER_MATCH);
     await action.finalize();
     is(action.lastError, null, "lastError should be null");
 
@@ -454,7 +479,7 @@ decorate_task(
     };
 
     const action = new PreferenceRolloutAction();
-    await action.runRecipe(recipe);
+    await action.processRecipe(recipe, BaseAction.suitability.FILTER_MATCH);
     await action.finalize();
     is(action.lastError, null, "lastError should be null");
 
@@ -469,8 +494,9 @@ decorate_task(
       "default branch value should change"
     );
 
+    const rollouts = await PreferenceRollouts.getAll();
     Assert.deepEqual(
-      await PreferenceRollouts.getAll(),
+      rollouts,
       [
         {
           slug: "test-rollout",
@@ -482,6 +508,7 @@ decorate_task(
               previousValue: "builtin value",
             },
           ],
+          enrollmentId: rollouts[0].enrollmentId,
         },
       ],
       "the rollout is added to the db with the correct previous value"
@@ -509,7 +536,7 @@ decorate_task(
     Services.prefs.setIntPref("test.pref", 2);
 
     const action = new PreferenceRolloutAction();
-    await action.runRecipe(recipe);
+    await action.processRecipe(recipe, BaseAction.suitability.FILTER_MATCH);
     await action.finalize();
     is(action.lastError, null, "lastError should be null");
 
@@ -550,22 +577,20 @@ decorate_task(
 
     // run once
     let action = new PreferenceRolloutAction();
-    await action.runRecipe(recipe);
+    await action.processRecipe(recipe, BaseAction.suitability.FILTER_MATCH);
     await action.finalize();
     is(action.lastError, null, "lastError should be null");
 
     // run a second time
     action = new PreferenceRolloutAction();
-    await action.runRecipe(recipe);
+    await action.processRecipe(recipe, BaseAction.suitability.FILTER_MATCH);
     await action.finalize();
     is(action.lastError, null, "lastError should be null");
 
-    sendEventStub.assertEvents([
-      ["enroll", "preference_rollout", "test-rollout"],
-    ]);
+    const rollouts = await PreferenceRollouts.getAll();
 
     Assert.deepEqual(
-      await PreferenceRollouts.getAll(),
+      rollouts,
       [
         {
           slug: "test-rollout",
@@ -573,10 +598,20 @@ decorate_task(
           preferences: [
             { preferenceName: "test.pref", value: 1, previousValue: null },
           ],
+          enrollmentId: rollouts[0].enrollmentId,
         },
       ],
       "the DB should have the correct value stored for previousValue"
     );
+
+    sendEventStub.assertEvents([
+      [
+        "enroll",
+        "preference_rollout",
+        "test-rollout",
+        { enrollmentId: rollouts[0].enrollmentId },
+      ],
+    ]);
   }
 );
 
@@ -597,7 +632,7 @@ decorate_task(
     };
 
     const action = new PreferenceRolloutAction();
-    await action.runRecipe(recipe);
+    await action.processRecipe(recipe, BaseAction.suitability.FILTER_MATCH);
     await action.finalize();
     is(action.lastError, null, "lastError should be null");
 
