@@ -428,7 +428,7 @@ nsresult nsNavHistory::TokensToQuery(const nsTArray<QueryKeyValuePair>& aTokens,
 
       // parent folders (guids)
     } else if (kvp.key.EqualsLiteral(QUERYKEY_PARENT)) {
-      NS_ENSURE_TRUE(parents.AppendElement(kvp.value), NS_ERROR_OUT_OF_MEMORY);
+      parents.AppendElement(kvp.value);
 
       // uri
     } else if (kvp.key.EqualsLiteral(QUERYKEY_URI)) {
@@ -462,7 +462,7 @@ nsresult nsNavHistory::TokensToQuery(const nsTArray<QueryKeyValuePair>& aTokens,
       NS_UnescapeURL(unescaped);  // modifies input
       NS_ConvertUTF8toUTF16 tag(unescaped);
       if (!tags.Contains(tag)) {
-        NS_ENSURE_TRUE(tags.AppendElement(tag), NS_ERROR_OUT_OF_MEMORY);
+        tags.AppendElement(tag);
       }
 
       // not tags
@@ -474,8 +474,7 @@ nsresult nsNavHistory::TokensToQuery(const nsTArray<QueryKeyValuePair>& aTokens,
       uint32_t transition = kvp.value.ToInteger(&rv);
       if (NS_SUCCEEDED(rv)) {
         if (!transitions.Contains(transition))
-          NS_ENSURE_TRUE(transitions.AppendElement(transition),
-                         NS_ERROR_OUT_OF_MEMORY);
+          transitions.AppendElement(transition);
       } else {
         NS_WARNING("Invalid Int32 transition value.");
       }
@@ -532,8 +531,7 @@ nsresult nsNavHistory::TokensToQuery(const nsTArray<QueryKeyValuePair>& aTokens,
   }
 
   if (tags.Length() > 0) {
-    rv = aQuery->SetTags(tags);
-    NS_ENSURE_SUCCESS(rv, rv);
+    aQuery->SetTags(std::move(tags));
   }
 
   if (transitions.Length() > 0) {
@@ -598,10 +596,10 @@ nsNavHistoryQuery::nsNavHistoryQuery(const nsNavHistoryQuery& aOther)
       mUri(aOther.mUri),
       mAnnotationIsNot(aOther.mAnnotationIsNot),
       mAnnotation(aOther.mAnnotation),
-      mParents(aOther.mParents),
-      mTags(aOther.mTags),
+      mParents(aOther.mParents.Clone()),
+      mTags(aOther.mTags.Clone()),
       mTagsAreNot(aOther.mTagsAreNot),
-      mTransitions(aOther.mTransitions) {}
+      mTransitions(aOther.mTransitions.Clone()) {}
 
 NS_IMETHODIMP nsNavHistoryQuery::GetBeginTime(PRTime* aBeginTime) {
   *aBeginTime = mBeginTime;
@@ -852,11 +850,9 @@ NS_IMETHODIMP nsNavHistoryQuery::SetTags(nsIVariant* aTags) {
     // Don't store duplicate tags.  This isn't just to save memory or to be
     // fancy; the SQL that's built from the tags relies on no dupes.
     if (!mTags.Contains(tag)) {
-      if (!mTags.AppendElement(tag)) {
-        free(tags[i]);
-        free(tags);
-        return NS_ERROR_OUT_OF_MEMORY;
-      }
+      // XXX(Bug 1631371) Check if this should use a fallible operation as it
+      // pretended earlier.
+      mTags.AppendElement(tag);
     }
     free(tags[i]);
   }
@@ -879,7 +875,7 @@ NS_IMETHODIMP nsNavHistoryQuery::SetTagsAreNot(bool aTagsAreNot) {
 }
 
 NS_IMETHODIMP nsNavHistoryQuery::GetParents(nsTArray<nsCString>& aGuids) {
-  aGuids = mParents;
+  aGuids = mParents.Clone();
   return NS_OK;
 }
 
@@ -899,7 +895,7 @@ NS_IMETHODIMP nsNavHistoryQuery::SetParents(const nsTArray<nsCString>& aGuids) {
 
 NS_IMETHODIMP nsNavHistoryQuery::GetTransitions(
     nsTArray<uint32_t>& aTransitions) {
-  aTransitions = mTransitions;
+  aTransitions = mTransitions.Clone();
   return NS_OK;
 }
 
