@@ -2,6 +2,7 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
+#include "js/Array.h"  // JS::GetArrayLength, JS::IsArrayObject
 #include "js/JSON.h"
 #include "jsapi.h"
 #include "mozilla/PresShell.h"
@@ -16,6 +17,7 @@
 #include "mozilla/dom/XPathResult.h"
 #include "mozilla/dom/XPathEvaluator.h"
 #include "mozilla/dom/XPathExpression.h"
+#include "mozilla/UniquePtr.h"
 #include "nsCharSeparatedTokenizer.h"
 #include "nsContentList.h"
 #include "nsContentUtils.h"
@@ -103,14 +105,14 @@ void SessionStoreUtils::ForEachNonDynamicChildFrame(
   }
 
   int32_t length;
-  aRv = docShell->GetChildCount(&length);
+  aRv = docShell->GetInProcessChildCount(&length);
   if (aRv.Failed()) {
     return;
   }
 
   for (int32_t i = 0; i < length; ++i) {
     nsCOMPtr<nsIDocShellTreeItem> item;
-    docShell->GetChildAt(i, getter_AddRefs(item));
+    docShell->GetInProcessChildAt(i, getter_AddRefs(item));
     if (!item) {
       aRv.Throw(NS_ERROR_FAILURE);
       return;
@@ -375,15 +377,189 @@ AppendEntryToCollectedData(nsINode* aNode, const nsAString& aId,
   return entry;
 }
 
-/*
-  @param aDocument: DOMDocument instance to obtain form data for.
-  @param aGeneratedCount: the current number of XPath expressions in the
-                          returned object.
-  @return aRetVal: Form data encoded in an object.
- */
-static void CollectFromTextAreaElement(Document& aDocument,
+// A helper function to append a element into aXPathVals or aIdVals
+static void AppendEntryToCollectedData(
+    nsINode* aNode, const nsAString& aId, CollectedInputDataValue& aEntry,
+    uint16_t& aNumXPath, uint16_t& aNumId,
+    nsTArray<CollectedInputDataValue>& aXPathVals,
+    nsTArray<CollectedInputDataValue>& aIdVals) {
+  if (!aId.IsEmpty()) {
+    aEntry.id = aId;
+    aIdVals.AppendElement(aEntry);
+    aNumId++;
+  } else {
+    nsAutoString xpath;
+    aNode->GenerateXPath(xpath);
+    aEntry.id = xpath;
+    aXPathVals.AppendElement(aEntry);
+    aNumXPath++;
+  }
+}
+
+/* for bool value */
+static void AppendValueToCollectedData(nsINode* aNode, const nsAString& aId,
+                                       const bool& aValue,
+                                       uint16_t& aGeneratedCount,
+                                       JSContext* aCx,
+                                       Nullable<CollectedData>& aRetVal) {
+  Record<nsString, OwningStringOrBooleanOrObject>::EntryType* entry =
+      AppendEntryToCollectedData(aNode, aId, aGeneratedCount, aRetVal);
+  entry->mValue.SetAsBoolean() = aValue;
+}
+
+/* for bool value */
+static void AppendValueToCollectedData(
+    nsINode* aNode, const nsAString& aId, const bool& aValue,
+    uint16_t& aNumXPath, uint16_t& aNumId,
+    nsTArray<CollectedInputDataValue>& aXPathVals,
+    nsTArray<CollectedInputDataValue>& aIdVals) {
+  CollectedInputDataValue entry;
+  entry.type = NS_LITERAL_STRING("bool");
+  entry.value = AsVariant(aValue);
+  AppendEntryToCollectedData(aNode, aId, entry, aNumXPath, aNumId, aXPathVals,
+                             aIdVals);
+}
+
+/* for nsString value */
+static void AppendValueToCollectedData(nsINode* aNode, const nsAString& aId,
+                                       const nsString& aValue,
                                        uint16_t& aGeneratedCount,
                                        Nullable<CollectedData>& aRetVal) {
+  Record<nsString, OwningStringOrBooleanOrObject>::EntryType* entry =
+      AppendEntryToCollectedData(aNode, aId, aGeneratedCount, aRetVal);
+  entry->mValue.SetAsString() = aValue;
+}
+
+/* for nsString value */
+static void AppendValueToCollectedData(
+    nsINode* aNode, const nsAString& aId, const nsString& aValue,
+    uint16_t& aNumXPath, uint16_t& aNumId,
+    nsTArray<CollectedInputDataValue>& aXPathVals,
+    nsTArray<CollectedInputDataValue>& aIdVals) {
+  CollectedInputDataValue entry;
+  entry.type = NS_LITERAL_STRING("string");
+  entry.value = AsVariant(aValue);
+  AppendEntryToCollectedData(aNode, aId, entry, aNumXPath, aNumId, aXPathVals,
+                             aIdVals);
+}
+
+/* for single select value */
+static void AppendValueToCollectedData(
+    nsINode* aNode, const nsAString& aId,
+    const CollectedNonMultipleSelectValue& aValue, uint16_t& aGeneratedCount,
+    JSContext* aCx, Nullable<CollectedData>& aRetVal) {
+  JS::Rooted<JS::Value> jsval(aCx);
+  if (!ToJSValue(aCx, aValue, &jsval)) {
+    JS_ClearPendingException(aCx);
+    return;
+  }
+  Record<nsString, OwningStringOrBooleanOrObject>::EntryType* entry =
+      AppendEntryToCollectedData(aNode, aId, aGeneratedCount, aRetVal);
+  entry->mValue.SetAsObject() = &jsval.toObject();
+}
+
+/* for single select value */
+static void AppendValueToCollectedData(
+    nsINode* aNode, const nsAString& aId,
+    const CollectedNonMultipleSelectValue& aValue, uint16_t& aNumXPath,
+    uint16_t& aNumId, nsTArray<CollectedInputDataValue>& aXPathVals,
+    nsTArray<CollectedInputDataValue>& aIdVals) {
+  CollectedInputDataValue entry;
+  entry.type = NS_LITERAL_STRING("singleSelect");
+  entry.value = AsVariant(aValue);
+  AppendEntryToCollectedData(aNode, aId, entry, aNumXPath, aNumId, aXPathVals,
+                             aIdVals);
+}
+
+/* special handing for input element with string type */
+static void AppendValueToCollectedData(Document& aDocument, nsINode* aNode,
+                                       const nsAString& aId,
+                                       const nsString& aValue,
+                                       uint16_t& aGeneratedCount,
+                                       JSContext* aCx,
+                                       Nullable<CollectedData>& aRetVal) {
+  if (!aId.IsEmpty()) {
+    // We want to avoid saving data for about:sessionrestore as a string.
+    // Since it's stored in the form as stringified JSON, stringifying
+    // further causes an explosion of escape characters. cf. bug 467409
+    if (aId.EqualsLiteral("sessionData")) {
+      nsAutoCString url;
+      Unused << aDocument.GetDocumentURI()->GetSpecIgnoringRef(url);
+      if (url.EqualsLiteral("about:sessionrestore") ||
+          url.EqualsLiteral("about:welcomeback")) {
+        JS::Rooted<JS::Value> jsval(aCx);
+        if (JS_ParseJSON(aCx, aValue.get(), aValue.Length(), &jsval) &&
+            jsval.isObject()) {
+          Record<nsString, OwningStringOrBooleanOrObject>::EntryType* entry =
+              AppendEntryToCollectedData(aNode, aId, aGeneratedCount, aRetVal);
+          entry->mValue.SetAsObject() = &jsval.toObject();
+        } else {
+          JS_ClearPendingException(aCx);
+        }
+        return;
+      }
+    }
+  }
+  AppendValueToCollectedData(aNode, aId, aValue, aGeneratedCount, aRetVal);
+}
+
+static void AppendValueToCollectedData(
+    Document& aDocument, nsINode* aNode, const nsAString& aId,
+    const nsString& aValue, uint16_t& aNumXPath, uint16_t& aNumId,
+    nsTArray<CollectedInputDataValue>& aXPathVals,
+    nsTArray<CollectedInputDataValue>& aIdVals) {
+  CollectedInputDataValue entry;
+  entry.type = NS_LITERAL_STRING("string");
+  entry.value = AsVariant(aValue);
+  AppendEntryToCollectedData(aNode, aId, entry, aNumXPath, aNumId, aXPathVals,
+                             aIdVals);
+}
+
+/* for nsTArray<nsString>: file and multipleSelect */
+static void AppendValueToCollectedData(nsINode* aNode, const nsAString& aId,
+                                       const nsAString& aValueType,
+                                       nsTArray<nsString>& aValue,
+                                       uint16_t& aGeneratedCount,
+                                       JSContext* aCx,
+                                       Nullable<CollectedData>& aRetVal) {
+  JS::Rooted<JS::Value> jsval(aCx);
+  if (aValueType.EqualsLiteral("file")) {
+    CollectedFileListValue val;
+    val.mType = aValueType;
+    val.mFileList.SwapElements(aValue);
+    if (!ToJSValue(aCx, val, &jsval)) {
+      JS_ClearPendingException(aCx);
+      return;
+    }
+  } else {
+    if (!ToJSValue(aCx, aValue, &jsval)) {
+      JS_ClearPendingException(aCx);
+      return;
+    }
+  }
+  Record<nsString, OwningStringOrBooleanOrObject>::EntryType* entry =
+      AppendEntryToCollectedData(aNode, aId, aGeneratedCount, aRetVal);
+  entry->mValue.SetAsObject() = &jsval.toObject();
+}
+
+/* for nsTArray<nsString>: file and multipleSelect */
+static void AppendValueToCollectedData(
+    nsINode* aNode, const nsAString& aId, const nsAString& aValueType,
+    const nsTArray<nsString>& aValue, uint16_t& aNumXPath, uint16_t& aNumId,
+    nsTArray<CollectedInputDataValue>& aXPathVals,
+    nsTArray<CollectedInputDataValue>& aIdVals) {
+  CollectedInputDataValue entry;
+  entry.type = aValueType;
+  entry.value = AsVariant(CopyableTArray(aValue.Clone()));
+  AppendEntryToCollectedData(aNode, aId, entry, aNumXPath, aNumId, aXPathVals,
+                             aIdVals);
+}
+
+/* static */
+template <typename... ArgsT>
+void SessionStoreUtils::CollectFromTextAreaElement(Document& aDocument,
+                                                   uint16_t& aGeneratedCount,
+                                                   ArgsT&&... args) {
   RefPtr<nsContentList> textlist = NS_GetContentList(
       &aDocument, kNameSpaceID_XHTML, NS_LITERAL_STRING("textarea"));
   uint32_t length = textlist->Length(true);
@@ -405,7 +581,7 @@ static void CollectFromTextAreaElement(Document& aDocument,
     if (id.IsEmpty() && (aGeneratedCount > kMaxTraversedXPaths)) {
       continue;
     }
-    nsAutoString value;
+    nsString value;
     textArea->GetValue(value);
     // In order to reduce XPath generation (which is slow), we only save data
     // for form fields that have been changed. (cf. bug 537289)
@@ -413,21 +589,16 @@ static void CollectFromTextAreaElement(Document& aDocument,
                               eCaseMatters)) {
       continue;
     }
-    Record<nsString, OwningStringOrBooleanOrObject>::EntryType* entry =
-        AppendEntryToCollectedData(textArea, id, aGeneratedCount, aRetVal);
-    entry->mValue.SetAsString() = value;
+    AppendValueToCollectedData(textArea, id, value, aGeneratedCount,
+                               std::forward<ArgsT>(args)...);
   }
 }
 
-/*
-  @param aDocument: DOMDocument instance to obtain form data for.
-  @param aGeneratedCount: the current number of XPath expressions in the
-                          returned object.
-  @return aRetVal: Form data encoded in an object.
- */
-static void CollectFromInputElement(JSContext* aCx, Document& aDocument,
-                                    uint16_t& aGeneratedCount,
-                                    Nullable<CollectedData>& aRetVal) {
+/* static */
+template <typename... ArgsT>
+void SessionStoreUtils::CollectFromInputElement(Document& aDocument,
+                                                uint16_t& aGeneratedCount,
+                                                ArgsT&&... args) {
   RefPtr<nsContentList> inputlist = NS_GetContentList(
       &aDocument, kNameSpaceID_XHTML, NS_LITERAL_STRING("input"));
   uint32_t length = inputlist->Length(true);
@@ -461,16 +632,15 @@ static void CollectFromInputElement(JSContext* aCx, Document& aDocument,
     if (!aInfo.IsNull() && !aInfo.Value().mCanAutomaticallyPersist) {
       continue;
     }
-    nsAutoString value;
+
     if (input->ControlType() == NS_FORM_INPUT_CHECKBOX ||
         input->ControlType() == NS_FORM_INPUT_RADIO) {
       bool checked = input->Checked();
       if (checked == input->DefaultChecked()) {
         continue;
       }
-      Record<nsString, OwningStringOrBooleanOrObject>::EntryType* entry =
-          AppendEntryToCollectedData(input, id, aGeneratedCount, aRetVal);
-      entry->mValue.SetAsBoolean() = checked;
+      AppendValueToCollectedData(input, id, checked, aGeneratedCount,
+                                 std::forward<ArgsT>(args)...);
     } else if (input->ControlType() == NS_FORM_INPUT_FILE) {
       IgnoredErrorResult rv;
       nsTArray<nsString> result;
@@ -478,19 +648,10 @@ static void CollectFromInputElement(JSContext* aCx, Document& aDocument,
       if (rv.Failed() || result.Length() == 0) {
         continue;
       }
-      CollectedFileListValue val;
-      val.mType = NS_LITERAL_STRING("file");
-      val.mFileList.SwapElements(result);
-
-      JS::Rooted<JS::Value> jsval(aCx);
-      if (!ToJSValue(aCx, val, &jsval)) {
-        JS_ClearPendingException(aCx);
-        continue;
-      }
-      Record<nsString, OwningStringOrBooleanOrObject>::EntryType* entry =
-          AppendEntryToCollectedData(input, id, aGeneratedCount, aRetVal);
-      entry->mValue.SetAsObject() = &jsval.toObject();
+      AppendValueToCollectedData(input, id, NS_LITERAL_STRING("file"), result,
+                                 aGeneratedCount, std::forward<ArgsT>(args)...);
     } else {
+      nsString value;
       input->GetValue(value, CallerType::System);
       // In order to reduce XPath generation (which is slow), we only save data
       // for form fields that have been changed. (cf. bug 537289)
@@ -501,45 +662,17 @@ static void CollectFromInputElement(JSContext* aCx, Document& aDocument,
                              eCaseMatters)) {
         continue;
       }
-      if (!id.IsEmpty()) {
-        // We want to avoid saving data for about:sessionrestore as a string.
-        // Since it's stored in the form as stringified JSON, stringifying
-        // further causes an explosion of escape characters. cf. bug 467409
-        if (id.EqualsLiteral("sessionData")) {
-          nsAutoCString url;
-          Unused << aDocument.GetDocumentURI()->GetSpecIgnoringRef(url);
-          if (url.EqualsLiteral("about:sessionrestore") ||
-              url.EqualsLiteral("about:welcomeback")) {
-            JS::Rooted<JS::Value> jsval(aCx);
-            if (JS_ParseJSON(aCx, value.get(), value.Length(), &jsval) &&
-                jsval.isObject()) {
-              Record<nsString, OwningStringOrBooleanOrObject>::EntryType*
-                  entry = AppendEntryToCollectedData(input, id, aGeneratedCount,
-                                                     aRetVal);
-              entry->mValue.SetAsObject() = &jsval.toObject();
-            } else {
-              JS_ClearPendingException(aCx);
-            }
-            continue;
-          }
-        }
-      }
-      Record<nsString, OwningStringOrBooleanOrObject>::EntryType* entry =
-          AppendEntryToCollectedData(input, id, aGeneratedCount, aRetVal);
-      entry->mValue.SetAsString() = value;
+      AppendValueToCollectedData(aDocument, input, id, value, aGeneratedCount,
+                                 std::forward<ArgsT>(args)...);
     }
   }
 }
 
-/*
-  @param aDocument: DOMDocument instance to obtain form data for.
-  @param aGeneratedCount: the current number of XPath expressions in the
-                          returned object.
-  @return aRetVal: Form data encoded in an object.
- */
-static void CollectFromSelectElement(JSContext* aCx, Document& aDocument,
-                                     uint16_t& aGeneratedCount,
-                                     Nullable<CollectedData>& aRetVal) {
+/* static */
+template <typename... ArgsT>
+void SessionStoreUtils::CollectFromSelectElement(Document& aDocument,
+                                                 uint16_t& aGeneratedCount,
+                                                 ArgsT&&... args) {
   RefPtr<nsContentList> selectlist = NS_GetContentList(
       &aDocument, kNameSpaceID_XHTML, NS_LITERAL_STRING("select"));
   uint32_t length = selectlist->Length(true);
@@ -569,15 +702,8 @@ static void CollectFromSelectElement(JSContext* aCx, Document& aDocument,
       CollectedNonMultipleSelectValue val;
       val.mSelectedIndex = select->SelectedIndex();
       val.mValue = selectVal.AsAString();
-
-      JS::Rooted<JS::Value> jsval(aCx);
-      if (!ToJSValue(aCx, val, &jsval)) {
-        JS_ClearPendingException(aCx);
-        continue;
-      }
-      Record<nsString, OwningStringOrBooleanOrObject>::EntryType* entry =
-          AppendEntryToCollectedData(select, id, aGeneratedCount, aRetVal);
-      entry->mValue.SetAsObject() = &jsval.toObject();
+      AppendValueToCollectedData(select, id, val, aGeneratedCount,
+                                 std::forward<ArgsT>(args)...);
     } else {
       // <select>s with the multiple attribute are easier to determine the
       // default value since each <option> has a defaultSelected property
@@ -603,14 +729,10 @@ static void CollectFromSelectElement(JSContext* aCx, Document& aDocument,
       if (hasDefaultValue) {
         continue;
       }
-      JS::Rooted<JS::Value> jsval(aCx);
-      if (!ToJSValue(aCx, selectslist, &jsval)) {
-        JS_ClearPendingException(aCx);
-        continue;
-      }
-      Record<nsString, OwningStringOrBooleanOrObject>::EntryType* entry =
-          AppendEntryToCollectedData(select, id, aGeneratedCount, aRetVal);
-      entry->mValue.SetAsObject() = &jsval.toObject();
+
+      AppendValueToCollectedData(
+          select, id, NS_LITERAL_STRING("multipleSelect"), selectslist,
+          aGeneratedCount, std::forward<ArgsT>(args)...);
     }
   }
 }
@@ -619,11 +741,14 @@ static void CollectCurrentFormData(JSContext* aCx, Document& aDocument,
                                    Nullable<CollectedData>& aRetVal) {
   uint16_t generatedCount = 0;
   /* textarea element */
-  CollectFromTextAreaElement(aDocument, generatedCount, aRetVal);
+  SessionStoreUtils::CollectFromTextAreaElement(aDocument, generatedCount,
+                                                aRetVal);
   /* input element */
-  CollectFromInputElement(aCx, aDocument, generatedCount, aRetVal);
+  SessionStoreUtils::CollectFromInputElement(aDocument, generatedCount, aCx,
+                                             aRetVal);
   /* select element */
-  CollectFromSelectElement(aCx, aDocument, generatedCount, aRetVal);
+  SessionStoreUtils::CollectFromSelectElement(aDocument, generatedCount, aCx,
+                                              aRetVal);
 
   Element* bodyElement = aDocument.GetBody();
   if (aDocument.HasFlag(NODE_IS_EDITABLE) && bodyElement) {
@@ -780,13 +905,13 @@ static void SetElementAsObject(JSContext* aCx, Element* aElement,
 
     // For Multiple Selects Element
     bool isArray = false;
-    JS_IsArrayObject(aCx, aObject, &isArray);
+    JS::IsArrayObject(aCx, aObject, &isArray);
     if (!isArray) {
       return;
     }
     JS::Rooted<JSObject*> arrayObj(aCx, &aObject.toObject());
     uint32_t arrayLength = 0;
-    if (!JS_GetArrayLength(aCx, arrayObj, &arrayLength)) {
+    if (!JS::GetArrayLength(aCx, arrayObj, &arrayLength)) {
       JS_ClearPendingException(aCx);
       return;
     }
@@ -867,7 +992,7 @@ static Element* FindNodeByXPath(JSContext* aCx, Document& aDocument,
                                 const nsAString& aExpression) {
   FormDataParseContext parsingContext(aDocument.IsHTMLDocument());
   IgnoredErrorResult rv;
-  nsAutoPtr<XPathExpression> expression(
+  UniquePtr<XPathExpression> expression(
       aDocument.XPathEvaluator()->CreateExpression(aExpression, &parsingContext,
                                                    &aDocument, rv));
   if (rv.Failed()) {
@@ -953,12 +1078,12 @@ bool SessionStoreUtils::RestoreFormData(const GlobalObject& aGlobal,
 }
 
 /* Read entries in the session storage data contained in a tab's history. */
-static void ReadAllEntriesFromStorage(
-    nsPIDOMWindowOuter* aWindow,
-    nsTHashtable<nsCStringHashKey>& aVisitedOrigins,
-    Record<nsString, Record<nsString, nsString>>& aRetVal) {
-  nsCOMPtr<nsIDocShell> docShell = aWindow->GetDocShell();
-  if (!docShell) {
+static void ReadAllEntriesFromStorage(nsPIDOMWindowOuter* aWindow,
+                                      nsTArray<nsCString>& aOrigins,
+                                      nsTArray<nsString>& aKeys,
+                                      nsTArray<nsString>& aValues) {
+  BrowsingContext* const browsingContext = aWindow->GetBrowsingContext();
+  if (!browsingContext) {
     return;
   }
 
@@ -972,20 +1097,21 @@ static void ReadAllEntriesFromStorage(
     return;
   }
 
-  nsCOMPtr<nsIPrincipal> storagePrincipal = doc->EffectiveStoragePrincipal();
+  nsCOMPtr<nsIPrincipal> storagePrincipal = doc->IntrinsicStoragePrincipal();
   if (!storagePrincipal) {
     return;
   }
 
   nsAutoCString origin;
-  nsresult rv = principal->GetOrigin(origin);
-  if (NS_FAILED(rv) || aVisitedOrigins.Contains(origin)) {
+  nsresult rv = storagePrincipal->GetOrigin(origin);
+  if (NS_FAILED(rv) || aOrigins.Contains(origin)) {
     // Don't read a host twice.
     return;
   }
 
   /* Completed checking for recursion and is about to read storage*/
-  nsCOMPtr<nsIDOMStorageManager> storageManager = do_QueryInterface(docShell);
+  const RefPtr<SessionStorageManager> storageManager =
+      browsingContext->GetSessionStorageManager();
   if (!storageManager) {
     return;
   }
@@ -1005,82 +1131,62 @@ static void ReadAllEntriesFromStorage(
     return;
   }
 
-  Record<nsString, Record<nsString, nsString>>::EntryType* recordEntry =
-      nullptr;
   for (uint32_t i = 0; i < len; i++) {
-    Record<nsString, nsString>::EntryType entry;
+    nsString key, value;
     mozilla::IgnoredErrorResult res;
-    storage->Key(i, entry.mKey, *principal, res);
+    storage->Key(i, key, *principal, res);
     if (res.Failed()) {
       continue;
     }
 
-    storage->GetItem(entry.mKey, entry.mValue, *principal, res);
+    storage->GetItem(key, value, *principal, res);
     if (res.Failed()) {
       continue;
     }
 
-    if (!recordEntry) {
-      recordEntry = aRetVal.Entries().AppendElement();
-      recordEntry->mKey = NS_ConvertUTF8toUTF16(origin);
-      aVisitedOrigins.PutEntry(origin);
-    }
-    recordEntry->mValue.Entries().AppendElement(std::move(entry));
+    aKeys.AppendElement(key);
+    aValues.AppendElement(value);
+    aOrigins.AppendElement(origin);
   }
 }
 
 /* Collect Collect session storage from current frame and all child frame */
-static void CollectedSessionStorageInternal(
-    JSContext* aCx, BrowsingContext* aBrowsingContext,
-    nsTHashtable<nsCStringHashKey>& aVisitedOrigins,
-    Record<nsString, Record<nsString, nsString>>& aRetVal) {
+/* static */
+void SessionStoreUtils::CollectedSessionStorage(
+    BrowsingContext* aBrowsingContext, nsTArray<nsCString>& aOrigins,
+    nsTArray<nsString>& aKeys, nsTArray<nsString>& aValues) {
   /* Collect session store from current frame */
   nsPIDOMWindowOuter* window = aBrowsingContext->GetDOMWindow();
   if (!window) {
     return;
   }
-  ReadAllEntriesFromStorage(window, aVisitedOrigins, aRetVal);
+  ReadAllEntriesFromStorage(window, aOrigins, aKeys, aValues);
 
   /* Collect session storage from all child frame */
   nsCOMPtr<nsIDocShell> docShell = window->GetDocShell();
   if (!docShell) {
     return;
   }
-  int32_t length;
-  nsresult rv = docShell->GetChildCount(&length);
-  if (NS_FAILED(rv)) {
-    return;
-  }
-  for (int32_t i = 0; i < length; ++i) {
-    nsCOMPtr<nsIDocShellTreeItem> item;
-    docShell->GetChildAt(i, getter_AddRefs(item));
-    if (!item) {
+
+  // This is not going to work for fission. Bug 1572084 for tracking it.
+  for (BrowsingContext* child : aBrowsingContext->Children()) {
+    window = child->GetDOMWindow();
+    if (!window) {
       return;
     }
-    nsCOMPtr<nsIDocShell> childDocShell(do_QueryInterface(item));
-    if (!childDocShell) {
+    docShell = window->GetDocShell();
+    if (!docShell) {
       return;
     }
     bool isDynamic = false;
-    rv = childDocShell->GetCreatedDynamically(&isDynamic);
+    nsresult rv = docShell->GetCreatedDynamically(&isDynamic);
     if (NS_SUCCEEDED(rv) && isDynamic) {
       continue;
     }
-    CollectedSessionStorageInternal(
-        aCx, nsDocShell::Cast(childDocShell)->GetBrowsingContext(),
-        aVisitedOrigins, aRetVal);
+    SessionStoreUtils::CollectedSessionStorage(child, aOrigins, aKeys, aValues);
   }
 }
 
-/* static */
-void SessionStoreUtils::CollectSessionStorage(
-    const GlobalObject& aGlobal, WindowProxyHolder& aWindow,
-    Record<nsString, Record<nsString, nsString>>& aRetVal) {
-  nsTHashtable<nsCStringHashKey> visitedOrigins;
-  CollectedSessionStorageInternal(aGlobal.Context(), aWindow.get(),
-                                  visitedOrigins, aRetVal);
-}
-
 /* static */
 void SessionStoreUtils::RestoreSessionStorage(
     const GlobalObject& aGlobal, nsIDocShell* aDocShell,
@@ -1104,12 +1210,21 @@ void SessionStoreUtils::RestoreSessionStorage(
 
     // OriginAttributes are always after a '^' character
     int32_t pos = entry.mKey.RFindChar('^');
-    nsCOMPtr<nsIPrincipal> principal = BasePrincipal::CreateCodebasePrincipal(
+    nsCOMPtr<nsIPrincipal> principal = BasePrincipal::CreateContentPrincipal(
         NS_ConvertUTF16toUTF8(Substring(entry.mKey, 0, pos)));
-    nsresult rv;
-    nsCOMPtr<nsIDOMStorageManager> storageManager =
-        do_QueryInterface(aDocShell, &rv);
-    if (NS_FAILED(rv)) {
+    BrowsingContext* const browsingContext =
+        nsDocShell::Cast(aDocShell)->GetBrowsingContext();
+    if (!browsingContext) {
+      return;
+    }
+
+    nsCOMPtr<nsIPrincipal> storagePrincipal =
+        BasePrincipal::CreateContentPrincipal(
+            NS_ConvertUTF16toUTF8(entry.mKey));
+
+    const RefPtr<SessionStorageManager> storageManager =
+        browsingContext->GetSessionStorageManager();
+    if (!storageManager) {
       return;
     }
     RefPtr<Storage> storage;
@@ -1119,8 +1234,9 @@ void SessionStoreUtils::RestoreSessionStorage(
     // followup bug to bug 600307.
     // Null window because the current window doesn't match the principal yet
     // and loads about:blank.
-    storageManager->CreateStorage(nullptr, principal, principal, EmptyString(),
-                                  false, getter_AddRefs(storage));
+    storageManager->CreateStorage(nullptr, principal, storagePrincipal,
+                                  EmptyString(), false,
+                                  getter_AddRefs(storage));
     if (!storage) {
       continue;
     }
@@ -1168,11 +1284,10 @@ static void CollectFrameTreeData(JSContext* aCx,
   SequenceRooter<JSObject*> rooter(aCx, &childrenData);
   uint32_t trailingNullCounter = 0;
 
-  nsTArray<RefPtr<BrowsingContext>> children;
-  aBrowsingContext->GetChildren(children);
-  for (uint32_t i = 0; i < children.Length(); i++) {
+  // This is not going to work for fission. Bug 1572084 for tracking it.
+  for (auto& child : aBrowsingContext->Children()) {
     NullableRootedDictionary<CollectedData> data(aCx);
-    CollectFrameTreeData(aCx, children[i], data, aFunc);
+    CollectFrameTreeData(aCx, child, data, aFunc);
     if (data.IsNull()) {
       childrenData.AppendElement(nullptr);
       trailingNullCounter++;
@@ -1206,3 +1321,61 @@ static void CollectFrameTreeData(JSContext* aCx,
   CollectFrameTreeData(aGlobal.Context(), aWindow.get(), aRetVal,
                        CollectCurrentFormData);
 }
+
+/* static */ void SessionStoreUtils::ComposeInputData(
+    const nsTArray<CollectedInputDataValue>& aData, InputElementData& ret) {
+  nsTArray<int> selectedIndex, valueIdx;
+  nsTArray<nsString> id, selectVal, strVal, type;
+  nsTArray<bool> boolVal;
+
+  for (const CollectedInputDataValue& data : aData) {
+    id.AppendElement(data.id);
+    type.AppendElement(data.type);
+
+    if (data.value.is<mozilla::dom::CollectedNonMultipleSelectValue>()) {
+      valueIdx.AppendElement(selectVal.Length());
+      selectedIndex.AppendElement(
+          data.value.as<mozilla::dom::CollectedNonMultipleSelectValue>()
+              .mSelectedIndex);
+      selectVal.AppendElement(
+          data.value.as<mozilla::dom::CollectedNonMultipleSelectValue>()
+              .mValue);
+    } else if (data.value.is<CopyableTArray<nsString>>()) {
+      // The first valueIdx is "index of the first string value"
+      valueIdx.AppendElement(strVal.Length());
+      strVal.AppendElements(data.value.as<CopyableTArray<nsString>>());
+      // The second valueIdx is "index of the last string value" + 1
+      id.AppendElement(data.id);
+      type.AppendElement(data.type);
+      valueIdx.AppendElement(strVal.Length());
+    } else if (data.value.is<nsString>()) {
+      valueIdx.AppendElement(strVal.Length());
+      strVal.AppendElement(data.value.as<nsString>());
+    } else if (data.type.EqualsLiteral("bool")) {
+      valueIdx.AppendElement(boolVal.Length());
+      boolVal.AppendElement(data.value.as<bool>());
+    }
+  }
+
+  if (selectedIndex.Length() != 0) {
+    ret.mSelectedIndex.Construct(std::move(selectedIndex));
+  }
+  if (valueIdx.Length() != 0) {
+    ret.mValueIdx.Construct(std::move(valueIdx));
+  }
+  if (id.Length() != 0) {
+    ret.mId.Construct(std::move(id));
+  }
+  if (selectVal.Length() != 0) {
+    ret.mSelectVal.Construct(std::move(selectVal));
+  }
+  if (strVal.Length() != 0) {
+    ret.mStrVal.Construct(std::move(strVal));
+  }
+  if (type.Length() != 0) {
+    ret.mType.Construct(std::move(type));
+  }
+  if (boolVal.Length() != 0) {
+    ret.mBoolVal.Construct(std::move(boolVal));
+  }
+}
