@@ -28,11 +28,9 @@
 // (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 // OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
-
 #include "HashStore.h"
 #include "nsICryptoHash.h"
 #include "nsISeekableStream.h"
-#include "nsIStreamConverterService.h"
 #include "nsNetUtil.h"
 #include "nsCheckSummedOutputStream.h"
 #include "prio.h"
@@ -88,7 +86,7 @@
 //    byte sliced (numSubPrefixes)   uint32_t add chunk of SubPrefixes
 //    byte sliced (numSubPrefixes)   uint32_t sub chunk of SubPrefixes
 //    byte sliced (numSubPrefixes)   uint32_t SubPrefixes
-//    0...numAddCompletes            32-byte Completions + uint32_t addChunk
+//    byte sliced (numAddCompletes)  uint32_t add chunk of AddCompletes
 //    0...numSubCompletes            32-byte Completions + uint32_t addChunk
 //                                                       + uint32_t subChunk
 //    16-byte MD5 of all preceding data
@@ -98,29 +96,27 @@
 
 // MOZ_LOG=UrlClassifierDbService:5
 extern mozilla::LazyLogModule gUrlClassifierDbServiceLog;
-#define LOG(args) MOZ_LOG(gUrlClassifierDbServiceLog, mozilla::LogLevel::Debug, args)
-#define LOG_ENABLED() MOZ_LOG_TEST(gUrlClassifierDbServiceLog, mozilla::LogLevel::Debug)
+#define LOG(args) \
+  MOZ_LOG(gUrlClassifierDbServiceLog, mozilla::LogLevel::Debug, args)
+#define LOG_ENABLED() \
+  MOZ_LOG_TEST(gUrlClassifierDbServiceLog, mozilla::LogLevel::Debug)
 
-namespace mozilla {
-namespace safebrowsing {
+namespace mozilla::safebrowsing {
 
 const uint32_t STORE_MAGIC = 0x1231af3b;
-const uint32_t CURRENT_VERSION = 3;
+const uint32_t CURRENT_VERSION = 4;
 
-nsresult
-TableUpdateV2::NewAddPrefix(uint32_t aAddChunk, const Prefix& aHash)
-{
-  AddPrefix *add = mAddPrefixes.AppendElement(fallible);
+nsresult TableUpdateV2::NewAddPrefix(uint32_t aAddChunk, const Prefix& aHash) {
+  AddPrefix* add = mAddPrefixes.AppendElement(fallible);
   if (!add) return NS_ERROR_OUT_OF_MEMORY;
   add->addChunk = aAddChunk;
   add->prefix = aHash;
   return NS_OK;
 }
 
-nsresult
-TableUpdateV2::NewSubPrefix(uint32_t aAddChunk, const Prefix& aHash, uint32_t aSubChunk)
-{
-  SubPrefix *sub = mSubPrefixes.AppendElement(fallible);
+nsresult TableUpdateV2::NewSubPrefix(uint32_t aAddChunk, const Prefix& aHash,
+                                     uint32_t aSubChunk) {
+  SubPrefix* sub = mSubPrefixes.AppendElement(fallible);
   if (!sub) return NS_ERROR_OUT_OF_MEMORY;
   sub->addChunk = aAddChunk;
   sub->prefix = aHash;
@@ -128,20 +124,19 @@ TableUpdateV2::NewSubPrefix(uint32_t aAddChunk, const Prefix& aHash, uint32_t aS
   return NS_OK;
 }
 
-nsresult
-TableUpdateV2::NewAddComplete(uint32_t aAddChunk, const Completion& aHash)
-{
-  AddComplete *add = mAddCompletes.AppendElement(fallible);
+nsresult TableUpdateV2::NewAddComplete(uint32_t aAddChunk,
+                                       const Completion& aHash) {
+  AddComplete* add = mAddCompletes.AppendElement(fallible);
   if (!add) return NS_ERROR_OUT_OF_MEMORY;
   add->addChunk = aAddChunk;
   add->complete = aHash;
   return NS_OK;
 }
 
-nsresult
-TableUpdateV2::NewSubComplete(uint32_t aAddChunk, const Completion& aHash, uint32_t aSubChunk)
-{
-  SubComplete *sub = mSubCompletes.AppendElement(fallible);
+nsresult TableUpdateV2::NewSubComplete(uint32_t aAddChunk,
+                                       const Completion& aHash,
+                                       uint32_t aSubChunk) {
+  SubComplete* sub = mSubCompletes.AppendElement(fallible);
   if (!sub) return NS_ERROR_OUT_OF_MEMORY;
   sub->addChunk = aAddChunk;
   sub->complete = aHash;
@@ -149,22 +144,18 @@ TableUpdateV2::NewSubComplete(uint32_t aAddChunk, const Completion& aHash, uint3
   return NS_OK;
 }
 
-nsresult
-TableUpdateV2::NewMissPrefix(const Prefix& aPrefix)
-{
-  Prefix *prefix = mMissPrefixes.AppendElement(aPrefix, fallible);
+nsresult TableUpdateV2::NewMissPrefix(const Prefix& aPrefix) {
+  Prefix* prefix = mMissPrefixes.AppendElement(aPrefix, fallible);
   if (!prefix) return NS_ERROR_OUT_OF_MEMORY;
   return NS_OK;
 }
 
-void
-TableUpdateV4::NewPrefixes(int32_t aSize, std::string& aPrefixes)
-{
+void TableUpdateV4::NewPrefixes(int32_t aSize, const nsACString& aPrefixes) {
   NS_ENSURE_TRUE_VOID(aSize >= 4 && aSize <= COMPLETE_SIZE);
-  NS_ENSURE_TRUE_VOID(aPrefixes.size() % aSize == 0);
+  NS_ENSURE_TRUE_VOID(aPrefixes.Length() % aSize == 0);
   NS_ENSURE_TRUE_VOID(!mPrefixesMap.Get(aSize));
 
-  int numOfPrefixes = aPrefixes.size() / aSize;
+  int numOfPrefixes = aPrefixes.Length() / aSize;
 
   if (aSize > 4) {
     // TODO Bug 1364043 we may have a better API to record multiple samples into
@@ -175,32 +166,33 @@ TableUpdateV4::NewPrefixes(int32_t aSize, std::string& aPrefixes)
     }
 #endif
   } else if (LOG_ENABLED()) {
-    uint32_t* p = (uint32_t*)aPrefixes.c_str();
+    const uint32_t* p =
+        reinterpret_cast<const uint32_t*>(ToNewCString(aPrefixes));
 
     // Dump the first/last 10 fixed-length prefixes for debugging.
     LOG(("* The first 10 (maximum) fixed-length prefixes: "));
     for (int i = 0; i < std::min(10, numOfPrefixes); i++) {
-      uint8_t* c = (uint8_t*)&p[i];
+      const uint8_t* c = reinterpret_cast<const uint8_t*>(&p[i]);
       LOG(("%.2X%.2X%.2X%.2X", c[0], c[1], c[2], c[3]));
     }
 
     LOG(("* The last 10 (maximum) fixed-length prefixes: "));
     for (int i = std::max(0, numOfPrefixes - 10); i < numOfPrefixes; i++) {
-      uint8_t* c = (uint8_t*)&p[i];
+      const uint8_t* c = reinterpret_cast<const uint8_t*>(&p[i]);
       LOG(("%.2X%.2X%.2X%.2X", c[0], c[1], c[2], c[3]));
     }
 
-    LOG(("---- %zu fixed-length prefixes in total.", aPrefixes.size() / aSize));
+    LOG(("---- %u fixed-length prefixes in total.",
+         aPrefixes.Length() / aSize));
   }
 
-  PrefixStdString* prefix = new PrefixStdString(aPrefixes);
-  mPrefixesMap.Put(aSize, prefix);
+  mPrefixesMap.Put(aSize, new nsCString(aPrefixes));
 }
 
-nsresult
-TableUpdateV4::NewRemovalIndices(const uint32_t* aIndices, size_t aNumOfIndices)
-{
-  MOZ_ASSERT(mRemovalIndiceArray.IsEmpty(), "mRemovalIndiceArray must be empty");
+nsresult TableUpdateV4::NewRemovalIndices(const uint32_t* aIndices,
+                                          size_t aNumOfIndices) {
+  MOZ_ASSERT(mRemovalIndiceArray.IsEmpty(),
+             "mRemovalIndiceArray must be empty");
 
   if (!mRemovalIndiceArray.SetCapacity(aNumOfIndices, fallible)) {
     return NS_ERROR_OUT_OF_MEMORY;
@@ -212,18 +204,14 @@ TableUpdateV4::NewRemovalIndices(const uint32_t* aIndices, size_t aNumOfIndices)
   return NS_OK;
 }
 
-void
-TableUpdateV4::NewChecksum(const std::string& aChecksum)
-{
-  mChecksum.Assign(aChecksum.data(), aChecksum.size());
+void TableUpdateV4::SetSHA256(const std::string& aSHA256) {
+  mSHA256.Assign(aSHA256.data(), aSHA256.size());
 }
 
-nsresult
-TableUpdateV4::NewFullHashResponse(const Prefix& aPrefix,
-                                   CachedFullHashResponse& aResponse)
-{
+nsresult TableUpdateV4::NewFullHashResponse(
+    const Prefix& aPrefix, const CachedFullHashResponse& aResponse) {
   CachedFullHashResponse* response =
-    mFullHashResponseMap.LookupOrAdd(aPrefix.ToUint32());
+      mFullHashResponseMap.LookupOrAdd(aPrefix.ToUint32());
   if (!response) {
     return NS_ERROR_OUT_OF_MEMORY;
   }
@@ -231,31 +219,35 @@ TableUpdateV4::NewFullHashResponse(const Prefix& aPrefix,
   return NS_OK;
 }
 
-HashStore::HashStore(const nsACString& aTableName,
-                     const nsACString& aProvider,
+void TableUpdateV4::Clear() {
+  mPrefixesMap.Clear();
+  mRemovalIndiceArray.Clear();
+}
+
+HashStore::HashStore(const nsACString& aTableName, const nsACString& aProvider,
                      nsIFile* aRootStoreDir)
-  : mTableName(aTableName)
-  , mInUpdate(false)
-  , mFileSize(0)
-{
-  nsresult rv = Classifier::GetPrivateStoreDirectory(aRootStoreDir,
-                                                     aTableName,
-                                                     aProvider,
-                                                     getter_AddRefs(mStoreDirectory));
+    : mTableName(aTableName), mInUpdate(false), mFileSize(0) {
+  nsresult rv = Classifier::GetPrivateStoreDirectory(
+      aRootStoreDir, aTableName, aProvider, getter_AddRefs(mStoreDirectory));
   if (NS_FAILED(rv)) {
     LOG(("Failed to get private store directory for %s", mTableName.get()));
     mStoreDirectory = aRootStoreDir;
   }
 }
 
-HashStore::~HashStore()
-= default;
+HashStore::~HashStore() = default;
 
-nsresult
-HashStore::Reset()
-{
+nsresult HashStore::Reset() {
   LOG(("HashStore resetting"));
 
+  // Close InputStream before removing the file
+  if (mInputStream) {
+    nsresult rv = mInputStream->Close();
+    NS_ENSURE_SUCCESS(rv, rv);
+
+    mInputStream = nullptr;
+  }
+
   nsCOMPtr<nsIFile> storeFile;
   nsresult rv = mStoreDirectory->Clone(getter_AddRefs(storeFile));
   NS_ENSURE_SUCCESS(rv, rv);
@@ -271,9 +263,7 @@ HashStore::Reset()
   return NS_OK;
 }
 
-nsresult
-HashStore::CheckChecksum(uint32_t aFileSize)
-{
+nsresult HashStore::CheckChecksum(uint32_t aFileSize) {
   if (!mInputStream) {
     return NS_OK;
   }
@@ -282,37 +272,34 @@ HashStore::CheckChecksum(uint32_t aFileSize)
   // comparing the stored checksum to actual checksum of data
   nsAutoCString hash;
   nsAutoCString compareHash;
-  char *data;
   uint32_t read;
 
   nsresult rv = CalculateChecksum(hash, aFileSize, true);
   NS_ENSURE_SUCCESS(rv, rv);
 
-  compareHash.GetMutableData(&data, hash.Length());
+  compareHash.SetLength(hash.Length());
 
   if (hash.Length() > aFileSize) {
-    NS_WARNING("SafeBrowing file not long enough to store its hash");
+    NS_WARNING("SafeBrowsing file not long enough to store its hash");
     return NS_ERROR_FAILURE;
   }
   nsCOMPtr<nsISeekableStream> seekIn = do_QueryInterface(mInputStream);
   rv = seekIn->Seek(nsISeekableStream::NS_SEEK_SET, aFileSize - hash.Length());
   NS_ENSURE_SUCCESS(rv, rv);
 
-  rv = mInputStream->Read(data, hash.Length(), &read);
+  rv = mInputStream->Read(compareHash.BeginWriting(), hash.Length(), &read);
   NS_ENSURE_SUCCESS(rv, rv);
   NS_ASSERTION(read == hash.Length(), "Could not read hash bytes");
 
   if (!hash.Equals(compareHash)) {
-    NS_WARNING("Safebrowing file failed checksum.");
+    NS_WARNING("SafeBrowsing file failed checksum.");
     return NS_ERROR_FAILURE;
   }
 
   return NS_OK;
 }
 
-nsresult
-HashStore::Open()
-{
+nsresult HashStore::Open(uint32_t aVersion) {
   nsCOMPtr<nsIFile> storeFile;
   nsresult rv = mStoreDirectory->Clone(getter_AddRefs(storeFile));
   NS_ENSURE_SUCCESS(rv, rv);
@@ -344,17 +331,18 @@ HashStore::Open()
   NS_ENSURE_SUCCESS(rv, rv);
 
   rv = ReadHeader();
-  NS_ENSURE_SUCCESS(rv, rv);
+  if (NS_WARN_IF(NS_FAILED(rv))) {
+    LOG(("Failed to read header for %s", mTableName.get()));
+    return NS_ERROR_FILE_CORRUPTED;
+  }
 
-  rv = SanityCheck();
+  rv = SanityCheck(aVersion);
   NS_ENSURE_SUCCESS(rv, rv);
 
   return NS_OK;
 }
 
-nsresult
-HashStore::ReadHeader()
-{
+nsresult HashStore::ReadHeader() {
   if (!mInputStream) {
     UpdateHeader();
     return NS_OK;
@@ -364,38 +352,36 @@ HashStore::ReadHeader()
   nsresult rv = seekable->Seek(nsISeekableStream::NS_SEEK_SET, 0);
   NS_ENSURE_SUCCESS(rv, rv);
 
-  void *buffer = &mHeader;
-  rv = NS_ReadInputStreamToBuffer(mInputStream,
-                                  &buffer,
-                                  sizeof(Header));
+  void* buffer = &mHeader;
+  rv = NS_ReadInputStreamToBuffer(mInputStream, &buffer, sizeof(Header));
   NS_ENSURE_SUCCESS(rv, rv);
 
   return NS_OK;
 }
 
-nsresult
-HashStore::SanityCheck()
-{
-  if (mHeader.magic != STORE_MAGIC || mHeader.version != CURRENT_VERSION) {
+nsresult HashStore::SanityCheck(uint32_t aVersion) const {
+  const uint32_t VER = aVersion == 0 ? CURRENT_VERSION : aVersion;
+  if (mHeader.magic != STORE_MAGIC || mHeader.version != VER) {
     NS_WARNING("Unexpected header data in the store.");
-    return NS_ERROR_FAILURE;
+    // Version mismatch is also considered file corrupted,
+    // We need this error code to know if we should remove the file.
+    return NS_ERROR_FILE_CORRUPTED;
   }
 
   return NS_OK;
 }
 
-nsresult
-HashStore::CalculateChecksum(nsAutoCString& aChecksum,
-                             uint32_t aFileSize,
-                             bool aChecksumPresent)
-{
+nsresult HashStore::CalculateChecksum(nsAutoCString& aChecksum,
+                                      uint32_t aFileSize,
+                                      bool aChecksumPresent) {
   aChecksum.Truncate();
 
   // Reset mInputStream to start
   nsCOMPtr<nsISeekableStream> seekable = do_QueryInterface(mInputStream);
   nsresult rv = seekable->Seek(nsISeekableStream::NS_SEEK_SET, 0);
 
-  nsCOMPtr<nsICryptoHash> hash = do_CreateInstance(NS_CRYPTO_HASH_CONTRACTID, &rv);
+  nsCOMPtr<nsICryptoHash> hash =
+      do_CreateInstance(NS_CRYPTO_HASH_CONTRACTID, &rv);
   NS_ENSURE_SUCCESS(rv, rv);
 
   // Size of MD5 hash in bytes
@@ -425,9 +411,7 @@ HashStore::CalculateChecksum(nsAutoCString& aChecksum,
   return NS_OK;
 }
 
-void
-HashStore::UpdateHeader()
-{
+void HashStore::UpdateHeader() {
   mHeader.magic = STORE_MAGIC;
   mHeader.version = CURRENT_VERSION;
 
@@ -439,32 +423,29 @@ HashStore::UpdateHeader()
   mHeader.numSubCompletes = mSubCompletes.Length();
 }
 
-nsresult
-HashStore::ReadChunkNumbers()
-{
-  if (!mInputStream || AlreadyReadChunkNumbers()) {
+nsresult HashStore::ReadChunkNumbers() {
+  if (!mInputStream) {
     return NS_OK;
   }
 
   nsCOMPtr<nsISeekableStream> seekable = do_QueryInterface(mInputStream);
-  nsresult rv = seekable->Seek(nsISeekableStream::NS_SEEK_SET,
-                               sizeof(Header));
+  nsresult rv = seekable->Seek(nsISeekableStream::NS_SEEK_SET, sizeof(Header));
   NS_ENSURE_SUCCESS(rv, rv);
 
   rv = mAddChunks.Read(mInputStream, mHeader.numAddChunks);
   NS_ENSURE_SUCCESS(rv, rv);
-  NS_ASSERTION(mAddChunks.Length() == mHeader.numAddChunks, "Read the right amount of add chunks.");
+  NS_ASSERTION(mAddChunks.Length() == mHeader.numAddChunks,
+               "Read the right amount of add chunks.");
 
   rv = mSubChunks.Read(mInputStream, mHeader.numSubChunks);
   NS_ENSURE_SUCCESS(rv, rv);
-  NS_ASSERTION(mSubChunks.Length() == mHeader.numSubChunks, "Read the right amount of sub chunks.");
+  NS_ASSERTION(mSubChunks.Length() == mHeader.numSubChunks,
+               "Read the right amount of sub chunks.");
 
   return NS_OK;
 }
 
-nsresult
-HashStore::ReadHashes()
-{
+nsresult HashStore::ReadHashes() {
   if (!mInputStream) {
     // BeginUpdate has been called but Open hasn't initialized mInputStream,
     // because the existing HashStore is empty.
@@ -484,12 +465,7 @@ HashStore::ReadHashes()
   rv = ReadSubPrefixes();
   NS_ENSURE_SUCCESS(rv, rv);
 
-  // If completions was read before, then we are done here.
-  if (AlreadyReadCompletions()) {
-    return NS_OK;
-  }
-
-  rv = ReadTArray(mInputStream, &mAddCompletes, mHeader.numAddCompletes);
+  rv = ReadAddCompletes();
   NS_ENSURE_SUCCESS(rv, rv);
 
   rv = ReadTArray(mInputStream, &mSubCompletes, mHeader.numSubCompletes);
@@ -498,43 +474,7 @@ HashStore::ReadHashes()
   return NS_OK;
 }
 
-
-nsresult
-HashStore::ReadCompletions()
-{
-  if (!mInputStream || AlreadyReadCompletions()) {
-    return NS_OK;
-  }
-
-  nsCOMPtr<nsIFile> storeFile;
-  nsresult rv = mStoreDirectory->Clone(getter_AddRefs(storeFile));
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  rv = storeFile->AppendNative(mTableName + NS_LITERAL_CSTRING(STORE_SUFFIX));
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  uint32_t offset = mFileSize -
-                    sizeof(struct AddComplete) * mHeader.numAddCompletes -
-                    sizeof(struct SubComplete) * mHeader.numSubCompletes -
-                    nsCheckSummedOutputStream::CHECKSUM_SIZE;
-
-  nsCOMPtr<nsISeekableStream> seekable = do_QueryInterface(mInputStream);
-
-  rv = seekable->Seek(nsISeekableStream::NS_SEEK_SET, offset);
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  rv = ReadTArray(mInputStream, &mAddCompletes, mHeader.numAddCompletes);
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  rv = ReadTArray(mInputStream, &mSubCompletes, mHeader.numSubCompletes);
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  return NS_OK;
-}
-
-nsresult
-HashStore::PrepareForUpdate()
-{
+nsresult HashStore::PrepareForUpdate() {
   nsresult rv = CheckChecksum(mFileSize);
   NS_ENSURE_SUCCESS(rv, rv);
 
@@ -547,9 +487,7 @@ HashStore::PrepareForUpdate()
   return NS_OK;
 }
 
-nsresult
-HashStore::BeginUpdate()
-{
+nsresult HashStore::BeginUpdate() {
   // Check wether the file is corrupted and read the rest of the store
   // in memory.
   nsresult rv = PrepareForUpdate();
@@ -567,14 +505,11 @@ HashStore::BeginUpdate()
   return NS_OK;
 }
 
-template<class T>
-static nsresult
-Merge(ChunkSet* aStoreChunks,
-      FallibleTArray<T>* aStorePrefixes,
-      ChunkSet& aUpdateChunks,
-      FallibleTArray<T>& aUpdatePrefixes,
-      bool aAllowMerging = false)
-{
+template <class T>
+static nsresult Merge(ChunkSet* aStoreChunks, FallibleTArray<T>* aStorePrefixes,
+                      const ChunkSet& aUpdateChunks,
+                      FallibleTArray<T>& aUpdatePrefixes,
+                      bool aAllowMerging = false) {
   EntrySort(aUpdatePrefixes);
 
   auto storeIter = aStorePrefixes->begin();
@@ -589,8 +524,7 @@ Merge(ChunkSet* aStoreChunks,
     // merging completions, in which case we'll always already
     // have the chunk from the original prefix
     if (aStoreChunks->Has(updatePrefix.Chunk()))
-      if (!aAllowMerging)
-        continue;
+      if (!aAllowMerging) continue;
     // XXX: binary search for insertion point might be faster in common
     // case?
     while (storeIter < storeEnd && (storeIter->Compare(updatePrefix) < 0)) {
@@ -598,8 +532,7 @@ Merge(ChunkSet* aStoreChunks,
       storeIter++;
     }
     // no match, add
-    if (storeIter == storeEnd
-        || storeIter->Compare(updatePrefix) != 0) {
+    if (storeIter == storeEnd || storeIter->Compare(updatePrefix) != 0) {
       if (!adds.AppendElement(updatePrefix, fallible)) {
         return NS_ERROR_OUT_OF_MEMORY;
       }
@@ -618,45 +551,38 @@ Merge(ChunkSet* aStoreChunks,
   return NS_OK;
 }
 
-nsresult
-HashStore::ApplyUpdate(TableUpdate &aUpdate)
-{
-  auto updateV2 = TableUpdate::Cast<TableUpdateV2>(&aUpdate);
-  NS_ENSURE_TRUE(updateV2, NS_ERROR_FAILURE);
+nsresult HashStore::ApplyUpdate(RefPtr<TableUpdateV2> aUpdate) {
+  MOZ_ASSERT(mTableName.Equals(aUpdate->TableName()));
 
-  TableUpdateV2& update = *updateV2;
-
-  nsresult rv = mAddExpirations.Merge(update.AddExpirations());
+  nsresult rv = mAddExpirations.Merge(aUpdate->AddExpirations());
   NS_ENSURE_SUCCESS(rv, rv);
 
-  rv = mSubExpirations.Merge(update.SubExpirations());
+  rv = mSubExpirations.Merge(aUpdate->SubExpirations());
   NS_ENSURE_SUCCESS(rv, rv);
 
   rv = Expire();
   NS_ENSURE_SUCCESS(rv, rv);
 
-  rv = Merge(&mAddChunks, &mAddPrefixes,
-             update.AddChunks(), update.AddPrefixes());
+  rv = Merge(&mAddChunks, &mAddPrefixes, aUpdate->AddChunks(),
+             aUpdate->AddPrefixes());
   NS_ENSURE_SUCCESS(rv, rv);
 
-  rv = Merge(&mAddChunks, &mAddCompletes,
-             update.AddChunks(), update.AddCompletes(), true);
+  rv = Merge(&mAddChunks, &mAddCompletes, aUpdate->AddChunks(),
+             aUpdate->AddCompletes(), true);
   NS_ENSURE_SUCCESS(rv, rv);
 
-  rv = Merge(&mSubChunks, &mSubPrefixes,
-             update.SubChunks(), update.SubPrefixes());
+  rv = Merge(&mSubChunks, &mSubPrefixes, aUpdate->SubChunks(),
+             aUpdate->SubPrefixes());
   NS_ENSURE_SUCCESS(rv, rv);
 
-  rv = Merge(&mSubChunks, &mSubCompletes,
-             update.SubChunks(), update.SubCompletes(), true);
+  rv = Merge(&mSubChunks, &mSubCompletes, aUpdate->SubChunks(),
+             aUpdate->SubCompletes(), true);
   NS_ENSURE_SUCCESS(rv, rv);
 
   return NS_OK;
 }
 
-nsresult
-HashStore::Rebuild()
-{
+nsresult HashStore::Rebuild() {
   NS_ASSERTION(mInUpdate, "Must be in update to rebuild.");
 
   nsresult rv = ProcessSubs();
@@ -667,21 +593,8 @@ HashStore::Rebuild()
   return NS_OK;
 }
 
-void
-HashStore::ClearCompletes()
-{
-  NS_ASSERTION(mInUpdate, "Must be in update to clear completes.");
-
-  mAddCompletes.Clear();
-  mSubCompletes.Clear();
-
-  UpdateHeader();
-}
-
-template<class T>
-static void
-ExpireEntries(FallibleTArray<T>* aEntries, ChunkSet& aExpirations)
-{
+template <class T>
+static void ExpireEntries(FallibleTArray<T>* aEntries, ChunkSet& aExpirations) {
   auto addIter = aEntries->begin();
 
   for (const auto& entry : *aEntries) {
@@ -694,9 +607,7 @@ ExpireEntries(FallibleTArray<T>* aEntries, ChunkSet& aExpirations)
   aEntries->TruncateLength(addIter - aEntries->begin());
 }
 
-nsresult
-HashStore::Expire()
-{
+nsresult HashStore::Expire() {
   ExpireEntries(&mAddPrefixes, mAddExpirations);
   ExpireEntries(&mAddCompletes, mAddExpirations);
   ExpireEntries(&mSubPrefixes, mSubExpirations);
@@ -711,9 +622,8 @@ HashStore::Expire()
   return NS_OK;
 }
 
-template<class T>
-nsresult DeflateWriteTArray(nsIOutputStream* aStream, nsTArray<T>& aIn)
-{
+template <class T>
+nsresult DeflateWriteTArray(nsIOutputStream* aStream, nsTArray<T>& aIn) {
   uLongf insize = aIn.Length() * sizeof(T);
   uLongf outsize = compressBound(insize);
   FallibleTArray<char> outBuff;
@@ -721,10 +631,8 @@ nsresult DeflateWriteTArray(nsIOutputStream* aStream, nsTArray<T>& aIn)
     return NS_ERROR_OUT_OF_MEMORY;
   }
 
-  int zerr = compress(reinterpret_cast<Bytef*>(outBuff.Elements()),
-                      &outsize,
-                      reinterpret_cast<const Bytef*>(aIn.Elements()),
-                      insize);
+  int zerr = compress(reinterpret_cast<Bytef*>(outBuff.Elements()), &outsize,
+                      reinterpret_cast<const Bytef*>(aIn.Elements()), insize);
   if (zerr != Z_OK) {
     return NS_ERROR_FAILURE;
   }
@@ -735,7 +643,8 @@ nsresult DeflateWriteTArray(nsIOutputStream* aStream, nsTArray<T>& aIn)
   // Length of compressed data stream
   uint32_t dataLen = outBuff.Length();
   uint32_t written;
-  nsresult rv = aStream->Write(reinterpret_cast<char*>(&dataLen), sizeof(dataLen), &written);
+  nsresult rv = aStream->Write(reinterpret_cast<char*>(&dataLen),
+                               sizeof(dataLen), &written);
   NS_ENSURE_SUCCESS(rv, rv);
 
   NS_ASSERTION(written == sizeof(dataLen), "Error writing deflate length");
@@ -747,14 +656,13 @@ nsresult DeflateWriteTArray(nsIOutputStream* aStream, nsTArray<T>& aIn)
   return NS_OK;
 }
 
-template<class T>
+template <class T>
 nsresult InflateReadTArray(nsIInputStream* aStream, FallibleTArray<T>* aOut,
-                           uint32_t aExpectedSize)
-{
-
+                           uint32_t aExpectedSize) {
   uint32_t inLen;
   uint32_t read;
-  nsresult rv = aStream->Read(reinterpret_cast<char*>(&inLen), sizeof(inLen), &read);
+  nsresult rv =
+      aStream->Read(reinterpret_cast<char*>(&inLen), sizeof(inLen), &read);
   NS_ENSURE_SUCCESS(rv, rv);
 
   NS_ASSERTION(read == sizeof(inLen), "Error reading inflate length");
@@ -773,23 +681,22 @@ nsresult InflateReadTArray(nsIInputStream* aStream, FallibleTArray<T>* aOut,
     return NS_ERROR_OUT_OF_MEMORY;
   }
 
-  int zerr = uncompress(reinterpret_cast<Bytef*>(aOut->Elements()),
-                        &outsize,
-                        reinterpret_cast<const Bytef*>(inBuff.Elements()),
-                        insize);
+  int zerr =
+      uncompress(reinterpret_cast<Bytef*>(aOut->Elements()), &outsize,
+                 reinterpret_cast<const Bytef*>(inBuff.Elements()), insize);
   if (zerr != Z_OK) {
     return NS_ERROR_FAILURE;
   }
   LOG(("InflateReadTArray: %lu in %lu out", insize, outsize));
 
-  NS_ASSERTION(outsize == aExpectedSize * sizeof(T), "Decompression size mismatch");
+  NS_ASSERTION(outsize == aExpectedSize * sizeof(T),
+               "Decompression size mismatch");
 
   return NS_OK;
 }
 
-static nsresult
-ByteSliceWrite(nsIOutputStream* aOut, nsTArray<uint32_t>& aData)
-{
+static nsresult ByteSliceWrite(nsIOutputStream* aOut,
+                               nsTArray<uint32_t>& aData) {
   nsTArray<uint8_t> slice;
   uint32_t count = aData.Length();
 
@@ -835,9 +742,8 @@ ByteSliceWrite(nsIOutputStream* aOut, nsTArray<uint32_t>& aData)
   return NS_OK;
 }
 
-static nsresult
-ByteSliceRead(nsIInputStream* aInStream, FallibleTArray<uint32_t>* aData, uint32_t count)
-{
+static nsresult ByteSliceRead(nsIInputStream* aInStream,
+                              FallibleTArray<uint32_t>* aData, uint32_t count) {
   FallibleTArray<uint8_t> slice1;
   FallibleTArray<uint8_t> slice2;
   FallibleTArray<uint8_t> slice3;
@@ -860,19 +766,16 @@ ByteSliceRead(nsIInputStream* aInStream, FallibleTArray<uint32_t>* aData, uint32
   }
 
   for (uint32_t i = 0; i < count; i++) {
-    aData->AppendElement((slice1[i] << 24) |
-                           (slice2[i] << 16) |
-                           (slice3[i] << 8) |
-                           (slice4[i]),
-                         fallible);
+    // SetCapacity was just called, these cannot fail.
+    (void)aData->AppendElement(
+        (slice1[i] << 24) | (slice2[i] << 16) | (slice3[i] << 8) | (slice4[i]),
+        fallible);
   }
 
   return NS_OK;
 }
 
-nsresult
-HashStore::ReadAddPrefixes()
-{
+nsresult HashStore::ReadAddPrefixes() {
   FallibleTArray<uint32_t> chunks;
   uint32_t count = mHeader.numAddPrefixes;
 
@@ -883,7 +786,7 @@ HashStore::ReadAddPrefixes()
     return NS_ERROR_OUT_OF_MEMORY;
   }
   for (uint32_t i = 0; i < count; i++) {
-    AddPrefix *add = mAddPrefixes.AppendElement(fallible);
+    AddPrefix* add = mAddPrefixes.AppendElement(fallible);
     add->prefix.FromUint32(0);
     add->addChunk = chunks[i];
   }
@@ -891,9 +794,25 @@ HashStore::ReadAddPrefixes()
   return NS_OK;
 }
 
-nsresult
-HashStore::ReadSubPrefixes()
-{
+nsresult HashStore::ReadAddCompletes() {
+  FallibleTArray<uint32_t> chunks;
+  uint32_t count = mHeader.numAddCompletes;
+
+  nsresult rv = ByteSliceRead(mInputStream, &chunks, count);
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  if (!mAddCompletes.SetCapacity(count, fallible)) {
+    return NS_ERROR_OUT_OF_MEMORY;
+  }
+  for (uint32_t i = 0; i < count; i++) {
+    AddComplete* add = mAddCompletes.AppendElement(fallible);
+    add->addChunk = chunks[i];
+  }
+
+  return NS_OK;
+}
+
+nsresult HashStore::ReadSubPrefixes() {
   FallibleTArray<uint32_t> addchunks;
   FallibleTArray<uint32_t> subchunks;
   FallibleTArray<uint32_t> prefixes;
@@ -912,7 +831,7 @@ HashStore::ReadSubPrefixes()
     return NS_ERROR_OUT_OF_MEMORY;
   }
   for (uint32_t i = 0; i < count; i++) {
-    SubPrefix *sub = mSubPrefixes.AppendElement(fallible);
+    SubPrefix* sub = mSubPrefixes.AppendElement(fallible);
     sub->addChunk = addchunks[i];
     sub->prefix.FromUint32(prefixes[i]);
     sub->subChunk = subchunks[i];
@@ -922,9 +841,7 @@ HashStore::ReadSubPrefixes()
 }
 
 // Split up PrefixArray back into the constituents
-nsresult
-HashStore::WriteAddPrefixes(nsIOutputStream* aOut)
-{
+nsresult HashStore::WriteAddPrefixChunks(nsIOutputStream* aOut) {
   nsTArray<uint32_t> chunks;
   uint32_t count = mAddPrefixes.Length();
   if (!chunks.SetCapacity(count, fallible)) {
@@ -941,9 +858,24 @@ HashStore::WriteAddPrefixes(nsIOutputStream* aOut)
   return NS_OK;
 }
 
-nsresult
-HashStore::WriteSubPrefixes(nsIOutputStream* aOut)
-{
+nsresult HashStore::WriteAddCompleteChunks(nsIOutputStream* aOut) {
+  nsTArray<uint32_t> chunks;
+  uint32_t count = mAddCompletes.Length();
+  if (!chunks.SetCapacity(count, fallible)) {
+    return NS_ERROR_OUT_OF_MEMORY;
+  }
+
+  for (uint32_t i = 0; i < count; i++) {
+    chunks.AppendElement(mAddCompletes[i].Chunk());
+  }
+
+  nsresult rv = ByteSliceWrite(aOut, chunks);
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  return NS_OK;
+}
+
+nsresult HashStore::WriteSubPrefixes(nsIOutputStream* aOut) {
   nsTArray<uint32_t> addchunks;
   nsTArray<uint32_t> subchunks;
   nsTArray<uint32_t> prefixes;
@@ -972,9 +904,7 @@ HashStore::WriteSubPrefixes(nsIOutputStream* aOut)
   return NS_OK;
 }
 
-nsresult
-HashStore::WriteFile()
-{
+nsresult HashStore::WriteFile() {
   NS_ASSERTION(mInUpdate, "Must be in update to write database.");
   if (nsUrlClassifierDBService::ShutdownHasStarted()) {
     return NS_ERROR_ABORT;
@@ -1002,13 +932,13 @@ HashStore::WriteFile()
   NS_ENSURE_SUCCESS(rv, rv);
 
   // Write hashes.
-  rv = WriteAddPrefixes(out);
+  rv = WriteAddPrefixChunks(out);
   NS_ENSURE_SUCCESS(rv, rv);
 
   rv = WriteSubPrefixes(out);
   NS_ENSURE_SUCCESS(rv, rv);
 
-  rv = WriteTArray(out, mAddCompletes);
+  rv = WriteAddCompleteChunks(out);
   NS_ENSURE_SUCCESS(rv, rv);
 
   rv = WriteTArray(out, mSubCompletes);
@@ -1023,12 +953,38 @@ HashStore::WriteFile()
   return NS_OK;
 }
 
+nsresult HashStore::ReadCompletionsLegacyV3(AddCompleteArray& aCompletes) {
+  if (mHeader.version != 3) {
+    return NS_ERROR_FAILURE;
+  }
+
+  nsCOMPtr<nsIFile> storeFile;
+  nsresult rv = mStoreDirectory->Clone(getter_AddRefs(storeFile));
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  rv = storeFile->AppendNative(mTableName + NS_LITERAL_CSTRING(STORE_SUFFIX));
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  uint32_t offset = mFileSize -
+                    sizeof(struct AddComplete) * mHeader.numAddCompletes -
+                    sizeof(struct SubComplete) * mHeader.numSubCompletes -
+                    nsCheckSummedOutputStream::CHECKSUM_SIZE;
+
+  nsCOMPtr<nsISeekableStream> seekable = do_QueryInterface(mInputStream);
+
+  rv = seekable->Seek(nsISeekableStream::NS_SEEK_SET, offset);
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  rv = ReadTArray(mInputStream, &aCompletes, mHeader.numAddCompletes);
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  return NS_OK;
+}
+
 template <class T>
-static void
-Erase(FallibleTArray<T>* array,
-      typename nsTArray<T>::iterator& iterStart,
-      typename nsTArray<T>::iterator& iterEnd)
-{
+static void Erase(FallibleTArray<T>* array,
+                  typename nsTArray<T>::iterator& iterStart,
+                  typename nsTArray<T>::iterator& iterEnd) {
   uint32_t start = iterStart - array->begin();
   uint32_t count = iterEnd - iterStart;
 
@@ -1045,10 +1001,9 @@ Erase(FallibleTArray<T>* array,
 //
 // |predAS| provides add < sub, |predSA| provides sub < add, for the
 // tightest compare appropriate (see calls in SBProcessSubs).
-template<class TSub, class TAdd>
-static void
-KnockoutSubs(FallibleTArray<TSub>* aSubs, FallibleTArray<TAdd>* aAdds)
-{
+template <class TSub, class TAdd>
+static void KnockoutSubs(FallibleTArray<TSub>* aSubs,
+                         FallibleTArray<TAdd>* aAdds) {
   // Keep a pair of output iterators for writing kept items.  Due to
   // deletions, these may lag the main iterators.  Using erase() on
   // individual items would result in O(N^2) copies.  Using a list
@@ -1089,9 +1044,8 @@ KnockoutSubs(FallibleTArray<TSub>* aSubs, FallibleTArray<TAdd>* aAdds)
 // Remove items in |removes| from |fullHashes|.  |fullHashes| and
 // |removes| should be ordered by SBAddPrefix component.
 template <class T>
-static void
-RemoveMatchingPrefixes(const SubPrefixArray& aSubs, FallibleTArray<T>* aFullHashes)
-{
+static void RemoveMatchingPrefixes(const SubPrefixArray& aSubs,
+                                   FallibleTArray<T>* aFullHashes) {
   // Where to store kept items.
   auto out = aFullHashes->begin();
   auto hashIter = aFullHashes->begin();
@@ -1114,17 +1068,14 @@ RemoveMatchingPrefixes(const SubPrefixArray& aSubs, FallibleTArray<T>* aFullHash
       // Drop equal items, there may be multiple hits.
       do {
         ++hashIter;
-      } while (hashIter != hashEnd &&
-               !(removeIter->CompareAlt(*hashIter) < 0));
+      } while (hashIter != hashEnd && !(removeIter->CompareAlt(*hashIter) < 0));
       ++removeIter;
     }
   }
   Erase(aFullHashes, out, hashIter);
 }
 
-static void
-RemoveDeadSubPrefixes(SubPrefixArray& aSubs, ChunkSet& aAddChunks)
-{
+static void RemoveDeadSubPrefixes(SubPrefixArray& aSubs, ChunkSet& aAddChunks) {
   auto subIter = aSubs.begin();
 
   for (const auto& sub : aSubs) {
@@ -1144,8 +1095,7 @@ RemoveDeadSubPrefixes(SubPrefixArray& aSubs, ChunkSet& aAddChunks)
 
 #ifdef DEBUG
 template <class T>
-static void EnsureSorted(FallibleTArray<T>* aArray)
-{
+static void EnsureSorted(FallibleTArray<T>* aArray) {
   auto start = aArray->begin();
   auto end = aArray->end();
   auto iter = start;
@@ -1158,14 +1108,10 @@ static void EnsureSorted(FallibleTArray<T>* aArray)
       MOZ_ASSERT(iter->Compare(*previous) >= 0);
     }
   }
-
-  return;
 }
 #endif
 
-nsresult
-HashStore::ProcessSubs()
-{
+nsresult HashStore::ProcessSubs() {
 #ifdef DEBUG
   EnsureSorted(&mAddPrefixes);
   EnsureSorted(&mSubPrefixes);
@@ -1197,76 +1143,39 @@ HashStore::ProcessSubs()
   return NS_OK;
 }
 
-nsresult
-HashStore::AugmentAdds(const nsTArray<uint32_t>& aPrefixes)
-{
-  uint32_t cnt = aPrefixes.Length();
-  if (cnt != mAddPrefixes.Length()) {
-    LOG(("Amount of prefixes in cache not consistent with store (%zu vs %zu)",
-         aPrefixes.Length(), mAddPrefixes.Length()));
+nsresult HashStore::AugmentAdds(const nsTArray<uint32_t>& aPrefixes,
+                                const nsTArray<nsCString>& aCompletes) {
+  if (aPrefixes.Length() != mAddPrefixes.Length() ||
+      aCompletes.Length() != mAddCompletes.Length()) {
+    LOG((
+        "Amount of prefixes/completes in cache not consistent with store prefixes(%zu vs %zu), \
+          store completes(%zu vs %zu)",
+        aPrefixes.Length(), mAddPrefixes.Length(), aCompletes.Length(),
+        mAddCompletes.Length()));
     return NS_ERROR_FAILURE;
   }
-  for (uint32_t i = 0; i < cnt; i++) {
+
+  for (size_t i = 0; i < mAddPrefixes.Length(); i++) {
     mAddPrefixes[i].prefix.FromUint32(aPrefixes[i]);
   }
+
+  for (size_t i = 0; i < mAddCompletes.Length(); i++) {
+    mAddCompletes[i].complete.Assign(aCompletes[i]);
+  }
+
   return NS_OK;
 }
 
-ChunkSet&
-HashStore::AddChunks()
-{
+ChunkSet& HashStore::AddChunks() {
   ReadChunkNumbers();
 
   return mAddChunks;
 }
 
-ChunkSet&
-HashStore::SubChunks()
-{
+ChunkSet& HashStore::SubChunks() {
   ReadChunkNumbers();
 
   return mSubChunks;
 }
 
-AddCompleteArray&
-HashStore::AddCompletes()
-{
-  ReadCompletions();
-
-  return mAddCompletes;
-}
-
-SubCompleteArray&
-HashStore::SubCompletes()
-{
-  ReadCompletions();
-
-  return mSubCompletes;
-}
-
-bool
-HashStore::AlreadyReadChunkNumbers()
-{
-  // If there are chunks but chunk set not yet contains any data
-  // Then we haven't read chunk numbers.
-  if ((mHeader.numAddChunks != 0 && mAddChunks.Length() == 0) ||
-      (mHeader.numSubChunks != 0 && mSubChunks.Length() == 0)) {
-    return false;
-  }
-  return true;
-}
-
-bool
-HashStore::AlreadyReadCompletions()
-{
-  // If there are completions but completion set not yet contains any data
-  // Then we haven't read completions.
-  if ((mHeader.numAddCompletes != 0 && mAddCompletes.Length() == 0) ||
-      (mHeader.numSubCompletes != 0 && mSubCompletes.Length() == 0)) {
-    return false;
-  }
-  return true;
-}
-
-} // namespace safebrowsing
-} // namespace mozilla
+}  // namespace mozilla::safebrowsing
