@@ -5,6 +5,9 @@
 const { AddonTestUtils } = ChromeUtils.import(
   "resource://testing-common/AddonTestUtils.jsm"
 );
+const { ExtensionPermissions } = ChromeUtils.import(
+  "resource://gre/modules/ExtensionPermissions.jsm"
+);
 
 const SECUREROOT =
   "https://example.com/browser/toolkit/mozapps/extensions/test/xpinstall/";
@@ -120,7 +123,7 @@ async function waitForProgressNotification(
 
 function acceptAppMenuNotificationWhenShown(
   id,
-  { dismiss = false, checkIncognito = false } = {}
+  { dismiss = false, checkIncognito = false, incognitoChecked = false } = {}
 ) {
   const { AppMenuNotifications } = ChromeUtils.import(
     "resource://gre/modules/AppMenuNotifications.jsm"
@@ -160,10 +163,8 @@ function acceptAppMenuNotificationWhenShown(
         allowPrivate && !checkIncognito,
         "checkbox visibility is correct"
       );
-      is(checkbox.checked, false, "checkbox is not initially checked");
-      if (checkIncognito) {
-        checkbox.checked = true;
-      }
+      is(checkbox.checked, incognitoChecked, "checkbox is marked as expected");
+      checkbox.checked = checkIncognito;
 
       if (dismiss) {
         // Dismiss the panel by clicking on the appMenu button.
@@ -298,8 +299,9 @@ function setupRedirect(aSettings) {
 
 var TESTS = [
   async function test_disabledInstall() {
-    Services.prefs.setBoolPref("xpinstall.enabled", false);
-
+    SpecialPowers.pushPrefEnv({
+      set: [["xpinstall.enabled", false]],
+    });
     let notificationPromise = waitForNotification("xpinstall-disabled");
     let triggers = encodeURIComponent(
       JSON.stringify({
@@ -340,6 +342,7 @@ var TESTS = [
     BrowserTestUtils.removeTab(gBrowser.selectedTab);
     let installs = await AddonManager.getAllInstalls();
     is(installs.length, 0, "Shouldn't be any pending installs");
+    await SpecialPowers.popPrefEnv();
   },
 
   async function test_blockedInstall() {
@@ -394,7 +397,7 @@ var TESTS = [
     let addon = await AddonManager.getAddonByID(
       "amosigned-xpi@tests.mozilla.org"
     );
-    addon.uninstall();
+    await addon.uninstall();
 
     await BrowserTestUtils.removeTab(gBrowser.selectedTab);
   },
@@ -422,7 +425,7 @@ var TESTS = [
     let installs = await AddonManager.getAllInstalls();
     is(installs.length, 0, "Should be no pending installs");
 
-    let installPerm = Services.perms.testPermission(
+    let installPerm = PermissionTestUtils.testPermission(
       gBrowser.currentURI,
       "install"
     );
@@ -434,7 +437,7 @@ var TESTS = [
 
     await BrowserTestUtils.removeTab(gBrowser.selectedTab);
 
-    SitePermissions.remove(NetUtil.newURI(target), "install");
+    PermissionTestUtils.remove(target, "install");
   },
 
   async function test_permaBlockedInstallNoPrompt() {
@@ -445,11 +448,7 @@ var TESTS = [
     );
     let target = TESTROOT + "installtrigger.html?" + triggers;
 
-    SitePermissions.set(
-      NetUtil.newURI(target),
-      "install",
-      SitePermissions.BLOCK
-    );
+    PermissionTestUtils.add(target, "install", Services.perms.DENY_ACTION);
     await BrowserTestUtils.openNewForegroundTab(gBrowser, target);
 
     let panelOpened;
@@ -469,19 +468,22 @@ var TESTS = [
 
     await BrowserTestUtils.removeTab(gBrowser.selectedTab);
 
-    SitePermissions.remove(NetUtil.newURI(target), "install");
+    PermissionTestUtils.remove(target, "install");
   },
 
   async function test_whitelistedInstall() {
-    Services.prefs.setBoolPref(
-      "extensions.allowPrivateBrowsingByDefault",
-      false
-    );
+    SpecialPowers.pushPrefEnv({
+      set: [["extensions.allowPrivateBrowsingByDefault", false]],
+    });
+
     let originalTab = gBrowser.selectedTab;
     let tab;
     gBrowser.selectedTab = originalTab;
-    let pm = Services.perms;
-    pm.add(makeURI("http://example.com/"), "install", pm.ALLOW_ACTION);
+    PermissionTestUtils.add(
+      "http://example.com/",
+      "install",
+      Services.perms.ALLOW_ACTION
+    );
 
     let progressPromise = waitForProgressNotification();
     let dialogPromise = waitForInstallDialog();
@@ -530,17 +532,20 @@ var TESTS = [
       "private browsing permission was not granted"
     );
 
-    addon.uninstall();
+    await addon.uninstall();
 
-    Services.perms.remove(makeURI("http://example.com/"), "install");
+    PermissionTestUtils.remove("http://example.com/", "install");
 
     Services.prefs.clearUserPref("extensions.allowPrivateBrowsingByDefault");
     await removeTabAndWaitForNotificationClose();
   },
 
   async function test_failedDownload() {
-    let pm = Services.perms;
-    pm.add(makeURI("http://example.com/"), "install", pm.ALLOW_ACTION);
+    PermissionTestUtils.add(
+      "http://example.com/",
+      "install",
+      Services.perms.ALLOW_ACTION
+    );
 
     let progressPromise = waitForProgressNotification();
     let failPromise = waitForNotification("addon-install-failed");
@@ -563,13 +568,16 @@ var TESTS = [
       "Should have seen the right message"
     );
 
-    Services.perms.remove(makeURI("http://example.com/"), "install");
+    PermissionTestUtils.remove("http://example.com/", "install");
     await removeTabAndWaitForNotificationClose();
   },
 
   async function test_corruptFile() {
-    let pm = Services.perms;
-    pm.add(makeURI("http://example.com/"), "install", pm.ALLOW_ACTION);
+    PermissionTestUtils.add(
+      "http://example.com/",
+      "install",
+      Services.perms.ALLOW_ACTION
+    );
 
     let progressPromise = waitForProgressNotification();
     let failPromise = waitForNotification("addon-install-failed");
@@ -593,13 +601,16 @@ var TESTS = [
       "Should have seen the right message"
     );
 
-    Services.perms.remove(makeURI("http://example.com/"), "install");
+    PermissionTestUtils.remove("http://example.com/", "install");
     await removeTabAndWaitForNotificationClose();
   },
 
   async function test_incompatible() {
-    let pm = Services.perms;
-    pm.add(makeURI("http://example.com/"), "install", pm.ALLOW_ACTION);
+    PermissionTestUtils.add(
+      "http://example.com/",
+      "install",
+      Services.perms.ALLOW_ACTION
+    );
 
     let progressPromise = waitForProgressNotification();
     let failPromise = waitForNotification("addon-install-failed");
@@ -620,16 +631,14 @@ var TESTS = [
       "chrome://branding/locale/brand.properties"
     );
     let brandShortName = brandBundle.GetStringFromName("brandShortName");
-    let message = `XPI Test could not be installed because it is not compatible with ${brandShortName} ${
-      Services.appinfo.version
-    }.`;
+    let message = `XPI Test could not be installed because it is not compatible with ${brandShortName} ${Services.appinfo.version}.`;
     is(
       notification.getAttribute("label"),
       message,
       "Should have seen the right message"
     );
 
-    Services.perms.remove(makeURI("http://example.com/"), "install");
+    PermissionTestUtils.remove("http://example.com/", "install");
     await removeTabAndWaitForNotificationClose();
   },
 
@@ -674,10 +683,9 @@ var TESTS = [
   },
 
   async function test_urlBar() {
-    Services.prefs.setBoolPref(
-      "extensions.allowPrivateBrowsingByDefault",
-      false
-    );
+    SpecialPowers.pushPrefEnv({
+      set: [["extensions.allowPrivateBrowsingByDefault", false]],
+    });
     let progressPromise = waitForProgressNotification();
     let dialogPromise = waitForInstallDialog();
 
@@ -732,7 +740,7 @@ var TESTS = [
       { actionType: "privateBrowsingAllowed" }
     );
 
-    addon.uninstall();
+    await addon.uninstall();
 
     Services.prefs.clearUserPref("extensions.allowPrivateBrowsingByDefault");
 
@@ -807,8 +815,11 @@ var TESTS = [
   },
 
   async function test_cancel() {
-    let pm = Services.perms;
-    pm.add(makeURI("http://example.com/"), "install", pm.ALLOW_ACTION);
+    PermissionTestUtils.add(
+      "http://example.com/",
+      "install",
+      Services.perms.ALLOW_ACTION
+    );
 
     let notificationPromise = waitForNotification(PROGRESS_NOTIFICATION);
     let triggers = encodeURIComponent(
@@ -862,12 +873,15 @@ var TESTS = [
     let installs = await AddonManager.getAllInstalls();
     is(installs.length, 0, "Should be no pending install");
 
-    Services.perms.remove(makeURI("http://example.com/"), "install");
+    PermissionTestUtils.remove("http://example.com/", "install");
     BrowserTestUtils.removeTab(gBrowser.selectedTab);
   },
 
   async function test_failedSecurity() {
-    Services.prefs.setBoolPref(PREF_INSTALL_REQUIREBUILTINCERTS, false);
+    SpecialPowers.pushPrefEnv({
+      set: [[PREF_INSTALL_REQUIREBUILTINCERTS, false]],
+    });
+
     setupRedirect({
       Location: TESTROOT + "amosigned.xpi",
     });
@@ -925,8 +939,209 @@ var TESTS = [
       "Should have seen the install fail"
     );
 
-    Services.prefs.setBoolPref(PREF_INSTALL_REQUIREBUILTINCERTS, true);
     await removeTabAndWaitForNotificationClose();
+    await SpecialPowers.popPrefEnv();
+  },
+
+  async function test_incognito_checkbox() {
+    SpecialPowers.pushPrefEnv({
+      set: [["extensions.allowPrivateBrowsingByDefault", false]],
+    });
+    // Grant permission up front.
+    const permissionName = "internal:privateBrowsingAllowed";
+    let incognitoPermission = {
+      permissions: [permissionName],
+      origins: [],
+    };
+    await ExtensionPermissions.add(
+      "amosigned-xpi@tests.mozilla.org",
+      incognitoPermission
+    );
+
+    let progressPromise = waitForProgressNotification();
+    let dialogPromise = waitForInstallDialog();
+
+    gBrowser.selectedTab = BrowserTestUtils.addTab(gBrowser, "about:blank");
+    await BrowserTestUtils.browserLoaded(gBrowser.selectedBrowser);
+    gURLBar.value = TESTROOT + "amosigned.xpi";
+    gURLBar.focus();
+    EventUtils.synthesizeKey("KEY_Enter");
+
+    await progressPromise;
+    let installDialog = await dialogPromise;
+
+    let notificationPromise = acceptAppMenuNotificationWhenShown(
+      "addon-installed",
+      { incognitoChecked: true }
+    );
+    installDialog.button.click();
+    await notificationPromise;
+
+    let installs = await AddonManager.getAllInstalls();
+    is(installs.length, 0, "Should be no pending installs");
+
+    let addon = await AddonManager.getAddonByID(
+      "amosigned-xpi@tests.mozilla.org"
+    );
+    // The panel is reloading the addon due to the permission change, we need some way
+    // to wait for the reload to finish. addon.startupPromise doesn't do it for
+    // us, so we'll just restart again.
+    await AddonTestUtils.promiseWebExtensionStartup(
+      "amosigned-xpi@tests.mozilla.org"
+    );
+
+    // This addon should no longer have private browsing permission.
+    let policy = WebExtensionPolicy.getByID(addon.id);
+    ok(!policy.privateBrowsingAllowed, "private browsing permission removed");
+
+    // Verify that the expected telemetry event has been collected for the extension allowed on
+    // PB windows from the "post install" notification doorhanger.
+    assertActionAMTelemetryEvent(
+      [
+        {
+          method: "action",
+          object: "doorhanger",
+          value: "off",
+          extra: {
+            action: "privateBrowsingAllowed",
+            view: "postInstall",
+            addonId: addon.id,
+            type: "extension",
+          },
+        },
+      ],
+      "Expect telemetry events for privateBrowsingAllowed action",
+      { actionType: "privateBrowsingAllowed" }
+    );
+
+    await addon.uninstall();
+
+    await removeTabAndWaitForNotificationClose();
+    await SpecialPowers.popPrefEnv();
+  },
+
+  async function test_incognito_checkbox_new_window() {
+    SpecialPowers.pushPrefEnv({
+      set: [["extensions.allowPrivateBrowsingByDefault", false]],
+    });
+    let win = await BrowserTestUtils.openNewBrowserWindow();
+    await SimpleTest.promiseFocus(win);
+    // Grant permission up front.
+    const permissionName = "internal:privateBrowsingAllowed";
+    let incognitoPermission = {
+      permissions: [permissionName],
+      origins: [],
+    };
+    await ExtensionPermissions.add(
+      "amosigned-xpi@tests.mozilla.org",
+      incognitoPermission
+    );
+
+    let panelEventPromise = new Promise(resolve => {
+      win.PopupNotifications.panel.addEventListener(
+        "PanelUpdated",
+        function eventListener(e) {
+          if (e.detail.includes("addon-webext-permissions")) {
+            win.PopupNotifications.panel.removeEventListener(
+              "PanelUpdated",
+              eventListener
+            );
+            resolve();
+          }
+        }
+      );
+    });
+
+    win.gBrowser.selectedTab = BrowserTestUtils.addTab(
+      win.gBrowser,
+      "about:blank"
+    );
+    await BrowserTestUtils.browserLoaded(win.gBrowser.selectedBrowser);
+    win.gURLBar.value = TESTROOT + "amosigned.xpi";
+    win.gURLBar.focus();
+    EventUtils.synthesizeKey("KEY_Enter", {}, win);
+
+    await panelEventPromise;
+    await waitForTick();
+
+    let panel = win.PopupNotifications.panel;
+    let installDialog = panel.childNodes[0];
+
+    let notificationPromise = new Promise(resolve => {
+      function popupshown() {
+        let notification = win.AppMenuNotifications.activeNotification;
+        if (!notification) {
+          return;
+        }
+
+        ok(win.PanelUI.isNotificationPanelOpen, "notification panel open");
+
+        win.PanelUI.notificationPanel.removeEventListener(
+          "popupshown",
+          popupshown
+        );
+
+        let checkbox = win.document.getElementById("addon-incognito-checkbox");
+        ok(!checkbox.hidden, "checkbox visibility is correct");
+        ok(checkbox.checked, "checkbox is marked as expected");
+        checkbox.checked = false;
+
+        // Dismiss the panel by clicking the primary button.
+        let popupnotificationID = win.PanelUI._getPopupId(notification);
+        let popupnotification = win.document.getElementById(
+          popupnotificationID
+        );
+        popupnotification.button.click();
+        resolve();
+      }
+      win.document.addEventListener("popupshown", popupshown);
+    });
+
+    installDialog.button.click();
+
+    await notificationPromise;
+
+    let installs = await AddonManager.getAllInstalls();
+    is(installs.length, 0, "Should be no pending installs");
+
+    let addon = await AddonManager.getAddonByID(
+      "amosigned-xpi@tests.mozilla.org"
+    );
+    // The panel is reloading the addon due to the permission change, we need some way
+    // to wait for the reload to finish. addon.startupPromise doesn't do it for
+    // us, so we'll just restart again.
+    await AddonTestUtils.promiseWebExtensionStartup(
+      "amosigned-xpi@tests.mozilla.org"
+    );
+
+    // This addon should no longer have private browsing permission.
+    let policy = WebExtensionPolicy.getByID(addon.id);
+    ok(!policy.privateBrowsingAllowed, "private browsing permission removed");
+
+    // Verify that the expected telemetry event has been collected for the extension allowed on
+    // PB windows from the "post install" notification doorhanger.
+    assertActionAMTelemetryEvent(
+      [
+        {
+          method: "action",
+          object: "doorhanger",
+          value: "off",
+          extra: {
+            action: "privateBrowsingAllowed",
+            view: "postInstall",
+            addonId: addon.id,
+            type: "extension",
+          },
+        },
+      ],
+      "Expect telemetry events for privateBrowsingAllowed action",
+      { actionType: "privateBrowsingAllowed" }
+    );
+
+    await addon.uninstall();
+
+    await SpecialPowers.popPrefEnv();
+    await BrowserTestUtils.closeWindow(win);
   },
 ];
 
@@ -952,10 +1167,14 @@ var XPInstallObserver = {
 add_task(async function() {
   requestLongerTimeout(4);
 
-  Services.prefs.setBoolPref("extensions.logging.enabled", true);
-  Services.prefs.setBoolPref("extensions.strictCompatibility", true);
-  Services.prefs.setBoolPref("extensions.install.requireSecureOrigin", false);
-  Services.prefs.setIntPref("security.dialog_enable_delay", 0);
+  SpecialPowers.pushPrefEnv({
+    set: [
+      ["extensions.logging.enabled", true],
+      ["extensions.strictCompatibility", true],
+      ["extensions.install.requireSecureOrigin", false],
+      ["security.dialog_enable_delay", 0],
+    ],
+  });
 
   Services.obs.addObserver(XPInstallObserver, "addon-install-started");
   Services.obs.addObserver(XPInstallObserver, "addon-install-blocked");
@@ -970,12 +1189,6 @@ add_task(async function() {
       aInstall.cancel();
     });
 
-    Services.prefs.clearUserPref("extensions.logging.enabled");
-    Services.prefs.clearUserPref("extensions.strictCompatibility");
-    Services.prefs.clearUserPref("extensions.install.requireSecureOrigin");
-    Services.prefs.clearUserPref("security.dialog_enable_delay");
-    Services.prefs.clearUserPref("extensions.allowPrivateBrowsingByDefault");
-
     Services.obs.removeObserver(XPInstallObserver, "addon-install-started");
     Services.obs.removeObserver(XPInstallObserver, "addon-install-blocked");
     Services.obs.removeObserver(XPInstallObserver, "addon-install-failed");
