@@ -114,8 +114,8 @@ void RemoveProfileFiles(nsIToolkitProfile* aProfile, bool aInBackground) {
         lock->Unlock();
         // nsIProfileLock is not threadsafe so release our reference to it on
         // the main thread.
-        NS_ReleaseOnMainThreadSystemGroup(
-            "nsToolkitProfile::RemoveProfileFiles::Unlock", lock.forget());
+        NS_ReleaseOnMainThread("nsToolkitProfile::RemoveProfileFiles::Unlock",
+                               lock.forget());
 
         rv = rootDir->Remove(true);
         NS_ENSURE_SUCCESS_VOID(rv);
@@ -393,10 +393,7 @@ nsToolkitProfileService::nsToolkitProfileService()
       mUpdateChannel(MOZ_STRINGIFY(MOZ_UPDATE_CHANNEL)),
       mProfileDBExists(false),
       mProfileDBFileSize(0),
-      mProfileDBModifiedTime(0),
-      mInstallDBExists(false),
-      mInstallDBFileSize(0),
-      mInstallDBModifiedTime(0) {
+      mProfileDBModifiedTime(0) {
 #ifdef MOZ_DEV_EDITION
   mUseDevEditionProfile = true;
 #endif
@@ -495,7 +492,7 @@ bool nsToolkitProfileService::IsProfileForCurrentInstall(
   lastGreDirPath = lastGreDir->NativePath();
   currentGreDirPath = currentGreDir->NativePath();
   if (lastGreDirPath.Equals(currentGreDirPath,
-                            nsCaseInsensitiveStringComparator())) {
+                            nsCaseInsensitiveStringComparator)) {
     return true;
   }
 
@@ -507,12 +504,12 @@ bool nsToolkitProfileService::IsProfileForCurrentInstall(
   if (SUCCEEDED(hres)) {
     nsDependentString strPathX86(pathX86);
     if (!StringBeginsWith(currentGreDirPath, strPathX86,
-                          nsCaseInsensitiveStringComparator())) {
+                          nsCaseInsensitiveStringComparator)) {
       PWSTR path = nullptr;
       hres = SHGetKnownFolderPath(FOLDERID_ProgramFiles, 0, nullptr, &path);
       if (SUCCEEDED(hres)) {
         if (StringBeginsWith(currentGreDirPath, nsDependentString(path),
-                             nsCaseInsensitiveStringComparator())) {
+                             nsCaseInsensitiveStringComparator)) {
           currentGreDirPath.Replace(0, wcslen(path), strPathX86);
         }
       }
@@ -522,7 +519,7 @@ bool nsToolkitProfileService::IsProfileForCurrentInstall(
   CoTaskMemFree(pathX86);
 
   return lastGreDirPath.Equals(currentGreDirPath,
-                               nsCaseInsensitiveStringComparator());
+                               nsCaseInsensitiveStringComparator);
 #  endif
 #endif
 
@@ -691,12 +688,6 @@ nsToolkitProfileService::GetIsListOutdated(bool* aResult) {
     return NS_OK;
   }
 
-  if (IsFileOutdated(mInstallDBFile, mInstallDBExists, mInstallDBModifiedTime,
-                     mInstallDBFileSize)) {
-    *aResult = true;
-    return NS_OK;
-  }
-
   *aResult = false;
   return NS_OK;
 }
@@ -750,10 +741,6 @@ nsresult nsToolkitProfileService::Init() {
   rv = mInstallDBFile->AppendNative(NS_LITERAL_CSTRING("installs.ini"));
   NS_ENSURE_SUCCESS(rv, rv);
 
-  rv = UpdateFileStats(mInstallDBFile, &mInstallDBExists,
-                       &mInstallDBModifiedTime, &mInstallDBFileSize);
-  NS_ENSURE_SUCCESS(rv, rv);
-
   nsAutoCString buffer;
 
   rv = UpdateFileStats(mProfileDBFile, &mProfileDBExists,
@@ -780,7 +767,7 @@ nsresult nsToolkitProfileService::Init() {
       // any install data from the backup.
       nsINIParser installDB;
 
-      if (mInstallDBExists && NS_SUCCEEDED(installDB.Init(mInstallDBFile))) {
+      if (NS_SUCCEEDED(installDB.Init(mInstallDBFile))) {
         // There is install data to import.
         ImportInstallsClosure closure = {&installDB, &mProfileDB};
         installDB.GetSections(&ImportInstalls, &closure);
@@ -1323,11 +1310,16 @@ nsresult nsToolkitProfileService::SelectStartupProfile(
 
     mStartupReason = NS_LITERAL_STRING("argument-profile");
 
-    // If a profile path is specified directly on the command line, then
-    // assume that the temp directory is the same as the given directory.
-    GetProfileByDir(lf, lf, getter_AddRefs(mCurrent));
+    GetProfileByDir(lf, nullptr, getter_AddRefs(mCurrent));
     NS_ADDREF(*aRootDir = lf);
-    lf.forget(aLocalDir);
+    // If the root dir matched a profile then use its local dir, otherwise use
+    // the root dir as the local dir.
+    if (mCurrent) {
+      mCurrent->GetLocalDir(aLocalDir);
+    } else {
+      lf.forget(aLocalDir);
+    }
+
     NS_IF_ADDREF(*aProfile = mCurrent);
     return NS_OK;
   }
@@ -1672,9 +1664,16 @@ void nsToolkitProfileService::GetProfileByDir(nsIFile* aRootDir,
     bool equal;
     nsresult rv = profile->mRootDir->Equals(aRootDir, &equal);
     if (NS_SUCCEEDED(rv) && equal) {
+      if (!aLocalDir) {
+        // If no local directory was given then we will just use the normal
+        // local directory for the profile.
+        profile.forget(aResult);
+        return;
+      }
+
       rv = profile->mLocalDir->Equals(aLocalDir, &equal);
       if (NS_SUCCEEDED(rv) && equal) {
-        NS_ADDREF(*aResult = profile);
+        profile.forget(aResult);
         return;
       }
     }
@@ -1978,17 +1977,12 @@ nsToolkitProfileService::Flush() {
       }
 
       fclose(writeFile);
-
-      rv = UpdateFileStats(mInstallDBFile, &mInstallDBExists,
-                           &mInstallDBModifiedTime, &mInstallDBFileSize);
-      NS_ENSURE_SUCCESS(rv, rv);
     } else {
       rv = mInstallDBFile->Remove(false);
       if (NS_FAILED(rv) && rv != NS_ERROR_FILE_TARGET_DOES_NOT_EXIST &&
           rv != NS_ERROR_FILE_NOT_FOUND) {
         return rv;
       }
-      mInstallDBExists = false;
     }
   }
 
